SystemOrganization addCategory: #FuelTests!
SystemOrganization addCategory: #'FuelTests-Collections'!
SystemOrganization addCategory: #'FuelTests-Mocks'!
SystemOrganization addCategory: #'FuelTests-StreamStrategies'!
SystemOrganization addCategory: #'FuelTests-Streams'!

Trait named: #FLTGlobalClassOrTraitSerializationTest
	uses: {}
	category: 'FuelTests'!

!FLTGlobalClassOrTraitSerializationTest commentStamp: '<historical>' prior: 0!
I test the serialization of classes and traits as *external* objects, i.e. the classes or traits have to be present in the image at materialization time.!

!FLTGlobalClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:59'!
testClassSideMethodNotFoundAfterRemoval
	"Tests that serializer does not tolarate when the method was removed between serialization and materialization"
	
	| classOrTrait |
	classOrTrait := self newClassOrTrait.
	classOrTrait classSide duringTestCompileSilently: 'you'.
	
	self serialize: classOrTrait classSide >> #you.
	classOrTrait classSide removeSelectorSilently: #you.
	
	self 
		should: [self materialized] 
		raise: FLMethodNotFound
		whoseDescriptionIncludes: classOrTrait classSide printString 
		description: 'Should raise an error when method is not present.' 
	
! !

!FLTGlobalClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:59'!
testClassSideMethodPreservesIdentity
	"Tests that serialization of a method in the class-side of a class or trait preserves identity"
	
	| classOrTrait |
	classOrTrait := self newClassOrTrait.
	classOrTrait classSide duringTestCompileSilently: 'you'.
	self assertSerializationIdentityOf: classOrTrait classSide >> #you! !

!FLTGlobalClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 19:38'!
testClassSideObsolete
	"Tests that serializer does not serialize as global the class-side of an obsolete class or trait."
	
	| classOrTrait |
	classOrTrait := self newClassOrTrait.
	self removeFromSystem: classOrTrait.

	self 
		should: [self serialize: classOrTrait ] 
		raise: FLObsolete
		whoseDescriptionIncludes: classOrTrait name
		description: 'Should raise an error when serializing as global an obsolete class or trait.' ! !

!FLTGlobalClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 11/28/2011 01:03'!
testClassSidePreservesIdentity
	"Tests that serialization of the class side preserves identity"
	
	self assertSerializationIdentityOf: self newClassOrTrait classSide ! !

!FLTGlobalClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:59'!
testMethodNotFoundAfterRemoval
	"Tests that serializer does not tolarate when the method was removed between serialization and materialization"
	
	| classOrTrait |
	classOrTrait := self newClassOrTrait.
	classOrTrait duringTestCompileSilently: 'you'.
	
	self serialize: classOrTrait >> #you.
	classOrTrait removeSelectorSilently: #you.
	
	self 
		should: [self materialized] 
		raise: FLMethodNotFound
		whoseDescriptionIncludes: classOrTrait name, '>>#you' 
		description: 'Should raise an error when method is not present.' 
	
! !

!FLTGlobalClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:59'!
testMethodPreservesIdentity
	"Tests that serialization of a method in a class or trait preserves identity"
	
	| classOrTrait |
	classOrTrait := self newClassOrTrait.
	classOrTrait duringTestCompileSilently: 'you'.
	self assertSerializationIdentityOf: classOrTrait >> #you! !

!FLTGlobalClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 19:37'!
testNotFoundAfterRemoval
	"Tests that serializer does not tolarate when the class was removed between serialization and materialization"
	
	| classOrTrait |
	classOrTrait := self newClassOrTrait.
	
	self serialize: classOrTrait.
	self removeFromSystem: classOrTrait.
	
	self should: [self materialized] raise: FLClassNotFound.! !

!FLTGlobalClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 18:47'!
testNotFoundAfterRename
	"Tests that serializer does not tolarate when the class was renamed between serialization and materialization"
	
	| classOrTrait |
	classOrTrait := self newClassOrTrait.
	
	self serialize: classOrTrait.
	classOrTrait renameSilently: (classOrTrait name, 'Renamed') asSymbol.
	
	self should: [self materialized] raise: FLClassNotFound.! !

!FLTGlobalClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 19:37'!
testObsolete
	"Tests that serializer does not serialize as global an obsolete class or trait."
	
	| classOrTrait |
	classOrTrait := self newClassOrTrait.
	self removeFromSystem: classOrTrait.

	self 
		should: [self serialize: classOrTrait ] 
		raise: FLObsolete
		whoseDescriptionIncludes: classOrTrait name
		description: 'Should raise an error when serializing as global an obsolete class or trait.' ! !

!FLTGlobalClassOrTraitSerializationTest methodsFor: 'tests' stamp: 'MartinDias 11/27/2011 21:08'!
testPreservesIdentity
	"Tests that serialization of the class or trait preserves identity"
	
	self assertSerializationIdentityOf: self newClassOrTrait! !

!Trait methodsFor: '*FuelTests' stamp: 'EstebanLorenzano 7/27/2012 16:30'!
renameSilently: aName
	SystemAnnouncer uniqueInstance doSilently: [
		self rename: aName.
	]! !

!CompiledMethod methodsFor: '*FuelTests' stamp: 'MarianoMartinezPeck 4/20/2012 21:18'!
isEqualRegardlessTrailerTo: aCompiledMethod
	^ (self copyWithTrailerBytes: CompiledMethodTrailer empty) = (aCompiledMethod copyWithTrailerBytes: CompiledMethodTrailer empty)! !

!CompiledMethod methodsFor: '*FuelTests' stamp: 'MarianoMartinezPeck 4/20/2012 21:23'!
sizeWithoutTrailer
	^ self trailer endPC! !

!ClassFactoryForTestCase methodsFor: '*FuelTests' stamp: 'MarianoMartinezPeck 4/19/2012 19:29'!
silentlyNewClassInCategory: category
	^self silentlyNewSubclassOf: self defaultSuperclass instanceVariableNames: '' classVariableNames: '' category: category! !

!ClassFactoryForTestCase methodsFor: '*FuelTests' stamp: 'MarianoMartinezPeck 4/19/2012 19:16'!
silentlyNewSubclassOf: aClass instanceVariableNames: ivNamesString classVariableNames:  classVarsString
	^ self silentlyNewSubclassOf: aClass instanceVariableNames: ivNamesString classVariableNames:  classVarsString category: self defaultCategoryPostfix
! !

!ClassFactoryForTestCase methodsFor: '*FuelTests' stamp: 'EstebanLorenzano 7/27/2012 16:30'!
silentlyNewSubclassOf: aClass instanceVariableNames: ivNamesString classVariableNames:  classVarsString category: category

	| default newClass |
	SystemAnnouncer uniqueInstance doSilently: [
		newClass := aClass 
			subclass: self newName 
			instanceVariableNames: ivNamesString 
			classVariableNames: classVarsString 
			poolDictionaries: '' 
			category: (self packageName, '-', category) asSymbol.
	].

	self createdClasses add: newClass.
	^newClass
	! !

!ClassFactoryForTestCase methodsFor: '*FuelTests' stamp: 'EstebanLorenzano 7/27/2012 16:30'!
silentlyNewSubclassOf: aClass instanceVariableNames: ivNamesString classVariableNames:  classVarsString package: packageName

	| default newClass |
	SystemAnnouncer uniqueInstance doSilently: [
		newClass := aClass 
			subclass: self newName 
			instanceVariableNames: ivNamesString 
			classVariableNames: classVarsString 
			poolDictionaries: '' 
			category: packageName asSymbol.
	].

	self createdClasses add: newClass.
	^newClass
	! !

!ClassFactoryForTestCase methodsFor: '*FuelTests' stamp: 'MarianoMartinezPeck 4/19/2012 19:06'!
withNotificationsNewClass
	^  self withNotificationsNewClassWithInstanceVariableNames: ''! !

!ClassFactoryForTestCase methodsFor: '*FuelTests' stamp: 'MarianoMartinezPeck 4/19/2012 19:23'!
withNotificationsNewClassWithInstanceVariableNames: instanceVariableNames
	^ self newSubclassOf: Object instanceVariableNames: instanceVariableNames classVariableNames:  '' category: self defaultCategoryPostfix
	! !

!MethodContext methodsFor: '*FuelTests' stamp: 'MarianoMartinezPeck 4/20/2012 21:40'!
assertWellMaterializedInto: aMethodContext in: aTestCase

	aTestCase assert: self ~~ aMethodContext. 
	aTestCase assert: (self class == aMethodContext class).
	aTestCase assert: self tempNames = aMethodContext  tempNames.
	aTestCase assert: pc = aMethodContext pc.
	aTestCase assert: stackp = aMethodContext stackPtr.
	closureOrNil isNil 
		ifTrue: [ aTestCase assert: aMethodContext closure isNil ]
		ifFalse: [ closureOrNil assertWellMaterializedInto: aMethodContext closure in: aTestCase ].
	aTestCase assert: receiver = aMethodContext receiver.
	aTestCase assert: (method isEqualRegardlessTrailerTo: aMethodContext method).
	sender isNil 
		ifTrue: [ aTestCase assert: aMethodContext sender isNil ]
		ifFalse: [ sender assertWellMaterializedInto: aMethodContext sender in: aTestCase ]! !

!MethodContext methodsFor: '*FuelTests' stamp: 'MarianoMartinezPeck 5/22/2011 23:07'!
initializeWith: aPc stackPtr: aStackPtr method: aMethod receiver: aReceiver sender: aSender
	pc := aPc.
	stackp := aStackPtr.
	method := aMethod.
	receiver := aReceiver.
	sender := aSender.
	! !

!TClassAndTraitDescription methodsFor: '*FuelTests' stamp: 'MarianoMartinezPeck 4/19/2012 12:46'!
duringTestCompileSilently: code

	^ Author 
		useAuthor: 'TestsAuthor'
		during: [ self compileSilently: code classified: '' ]! !

!TClassAndTraitDescription methodsFor: '*FuelTests' stamp: 'MarianoMartinezPeck 4/19/2012 12:48'!
duringTestCompileSilently: code classified: aCategory

	^ Author 
		useAuthor: 'TestsAuthor'
		during: [ self compileSilently: code classified: aCategory ]! !

!MethodDictionary methodsFor: '*FuelTests' stamp: 'MarianoMartinezPeck 4/20/2012 21:29'!
isEqualRegardlessMethodsTrailerTo: aMethodDictionary

	"Two dictionaries are equal if
	 (a) they are the same 'kind' of thing.
	 (b) they have the same set of keys.
	 (c) for each (common) key, they have the same value"

	self == aMethodDictionary ifTrue: [ ^ true ].
	(aMethodDictionary isDictionary) ifFalse: [^false].
	self size = aMethodDictionary size ifFalse: [^false].
	self associationsDo: [:assoc|
		((aMethodDictionary at: assoc key ifAbsent: [^false]) isEqualRegardlessTrailerTo:  assoc value)
			ifFalse: [^false]].
	^true

! !

!BlockClosure methodsFor: '*FuelTests' stamp: 'MarianoMartinezPeck 4/20/2012 21:16'!
assertWellMaterializedInto: aBlockClosure in: aTestCase
	
	aTestCase assert: self ~~ aBlockClosure. 
	aTestCase assert: (self class == aBlockClosure class).
	aTestCase assert: numArgs = aBlockClosure numArgs.
	aTestCase assert: startpc = aBlockClosure startpc.
	
	outerContext isNil
        ifTrue: [ self assert: aBlockClosure outerContext isNil ]
        ifFalse: [
            self isClean
                ifTrue: [    
                    "self assert: self receiver = aBlockClosure receiver."
                    self assert: (self method isEqualRegardlessTrailerTo: aBlockClosure method).
                    self assert: aBlockClosure outerContext sender isNil.
                    self assert: aBlockClosure outerContext arguments isEmpty.
                      ]
                ifFalse: [outerContext assertWellMaterializedInto: aBlockClosure outerContext in: aTestCase]
             ]
! !

TestCase subclass: #FLIndexStreamTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests-Streams'!

!FLIndexStreamTest methodsFor: 'running' stamp: 'MartinDias 12/30/2011 15:29'!
indexStreamOn: aStream digits: aNumberOfDigits
	^ FLIndexStream 
		on: aStream
		digits: aNumberOfDigits! !

!FLIndexStreamTest methodsFor: 'tests' stamp: 'MartinDias 12/30/2011 15:28'!
testCreation

	self 
		shouldnt: [ self indexStreamOn: #() writeStream digits: 2 ]
		raise: Error.! !

!FLIndexStreamTest methodsFor: 'tests' stamp: 'MartinDias 12/30/2011 15:28'!
testNextIndex

	| indexes indexStream targetStream |
	indexes := #(0 1 128 255).
	targetStream := #() writeStream.

	indexStream := self 
		indexStreamOn: targetStream
		digits: 1.
	indexes do: [:x | indexStream nextIndexPut: x ].

	indexStream := self 
		indexStreamOn: targetStream contents readStream
		digits: 1.

	indexes do: [:x | self assert: indexStream nextIndex = x ].! !

!FLIndexStreamTest methodsFor: 'tests' stamp: 'MartinDias 12/30/2011 15:28'!
testNextIndexPut

	| indexStream targetStream |
	targetStream := #() writeStream.
	indexStream := self 
		indexStreamOn: targetStream
		digits: 1.
	
	indexStream nextIndexPut: 0.
	indexStream nextIndexPut: 1.! !

TestCase subclass: #FLSerializationTest
	instanceVariableNames: 'serializer materializer classFactory streamFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

FLSerializationTest subclass: #FLBasicSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLBasicSerializationTest methodsFor: 'failures' stamp: 'MartinDias 6/25/2012 21:09'!
expectedFailures
	^ ((SystemVersion current major = 1) and: [SystemVersion current minor < 3]) 
		ifTrue: [
			#( #testSetElement #testSetWithNil #testSetWithSetElement )]
		ifFalse: [#(testConsiderCustomWideSymbolGlobal)]! !

!FLBasicSerializationTest methodsFor: 'tests-numbers' stamp: 'MarianoMartinezPeck 6/15/2012 19:55'!
testAllRangeOfIntegers

		self assertSerializationIdentityOf: 100.
		self assertSerializationIdentityOf: 10000.
		self assertSerializationIdentityOf: 100000.
		self assertSerializationIdentityOf: 10000000.
		self assertSerializationIdentityOf: 100000000.
		self assertSerializationIdentityOf: 1000000000.
		self assertSerializationEqualityOf: 3000000000.
		self assertSerializationEqualityOf: 10000000000.
		self assertSerializationEqualityOf: 100000000000.
		self assertSerializationEqualityOf: 100 factorial.

		self assertSerializationIdentityOf: -100.
		self assertSerializationIdentityOf: -10000.
		self assertSerializationIdentityOf: -100000.
		self assertSerializationIdentityOf: -10000000.
		self assertSerializationIdentityOf: -100000000.
		self assertSerializationIdentityOf: -1000000000.
		self assertSerializationEqualityOf: -3000000000.
		self assertSerializationEqualityOf: -10000000000.
		self assertSerializationEqualityOf: -100000000000.
		self assertSerializationEqualityOf:  100 factorial * -1.

! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MartinDias 9/19/2011 11:27'!
testArray
	self assertSerializationEqualityOf: #(1).
	self assertSerializationEqualityOf: #(10 20).
	self assertSerializationEqualityOf: #(1 2 3 #('Hello' 2 3)).
	

! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 9/6/2011 23:33'!
testAssociation

	self assertSerializationEqualityOf: 1-> 'marino'.
	self assertSerializationEqualityOf: nil-> 'marino'.
	self assertSerializationEqualityOf: nil-> nil.
	self assertSerializationEqualityOf: nil-> #(1 3 4).
	self assertSerializationEqualityOf: nil-> true.! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MartinDias 10/5/2011 17:54'!
testBag
	| bag materialized |
	bag := Bag new.
	bag add: 10.
	bag add: 20.
	bag add: 30.
	bag add: 30.
	
	materialized := self resultOfSerializeAndMaterialize: bag.
	self assert:  bag ~~ materialized.
	self assert: bag = materialized.
	self assert: materialized size = 4.
	! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MartinDias 10/5/2011 17:51'!
testBitmap
	| bitmap materialized |
	self assertSerializationEqualityOf: (Bitmap with: 0 with: 16rFFFFFFFF).
	self assertSerializationEqualityOf: (Bitmap with: 4278190080).
	self assertSerializationEqualityOf: (Bitmap new: 4096).
	self assertSerializationEqualityOf: (Bitmap with: 0 with: 4294967295).
	self assertSerializationEqualityOf: (Bitmap new: 256).
	self assertSerializationEqualityOf: (Bitmap with: 0).
	bitmap := Bitmap new: 3.
	bitmap at: 1 put: 4324.
	bitmap at: 2 put: 5674.
	bitmap at: 3 put: 8978.
	materialized := self resultOfSerializeAndMaterialize: bitmap.
	self assert: (materialized at: 1) = 4324.
	self assert: (materialized at: 2) = 5674.
	self assert: (materialized at: 3) = 8978.	


! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MarianoMartinezPeck 10/11/2011 12:00'!
testByteArrays

	self assertSerializationEqualityOf: { #[1 2 3 4 5 6]. #[1 2 3 4 5 6]. #[7 8 9 10 11 12]. #[7 8 9 10 11 12] }.
	self assertSerializationEqualityOf:  (ByteArray with: 10 with: 20 with: 30).
	self assertSerializationEqualityOf: ByteArray new.
	self assertSerializationEqualityOf: (ByteArray new: 100).
	self assertSerializationEqualityOf: (0 to: 255) asByteArray.! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 1/9/2012 21:00'!
testCharacter
	"Test character serialization. If the code is less than 255 the same instance is used. But if it is bigger, new ones are created. For more info read Character class comment and Character class >> value:"

	self assertSerializationIdentityOf: $a.
	self assertSerializationEqualityOf: (Character value: 12345). "Japanese Hiragana 'A' "
	self assertSerializationEqualityOf: Character allCharacters.
	self assertSerializationEqualityOf: (Array with: $a with: (Character value: 12345)).! !

!FLBasicSerializationTest methodsFor: 'tests-globals' stamp: 'MartinDias 12/15/2011 15:42'!
testClassSideCompiledMethod
	"They should be considered as globals by default."
	
	self assertSerializationIdentityOf: FLGlobalSendMock class >> #newInstanceToSerialize
! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 9/28/2011 17:40'!
testClasses
	"When we serialize this way, classes should be considered as globals."
	self assertSerializationEqualityOf: Collection withAllSubclasses asArray.
! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 6/20/2011 17:09'!
testColor

	self assertSerializationEqualityOf: Color blue.
	self assertSerializationEqualityOf: Color black.
	! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MartinDias 4/19/2011 13:02'!
testColorArray

	self assertSerializationEqualityOf: (ColorArray with: (Color r: 0.0 g: 0.031 b: 0.008) with: (Color r: 0.0 g: 0.07 b: 0.023))! !

!FLBasicSerializationTest methodsFor: 'tests-globals' stamp: 'MartinDias 12/15/2011 15:41'!
testCompiledMethod
	"They should be considered as globals by default."
	
	self assertSerializationIdentityOf: FLPair >> #left
! !

!FLBasicSerializationTest methodsFor: 'tests-globals' stamp: 'MartinDias 1/10/2012 14:29'!
testConsiderCustomGlobal
	"A custom global variable is treated as global by Fuel, when we explicitly specify this."
	
	| aPerson |
	self analyzer considerGlobal: #FLGlobalVariableForTesting.

	aPerson := FLPerson new.
	Smalltalk globals at: #FLGlobalVariableForTesting put: aPerson.
	
	self assertSerializationIdentityOf: aPerson.
! !

!FLBasicSerializationTest methodsFor: 'tests-globals' stamp: 'MartinDias 6/25/2012 21:08'!
testConsiderCustomWideSymbolGlobal
	"The same than #testConsiderCustomGlobal but with a WideSymbol."
	
	| aWideSymbol aPerson |
	aWideSymbol := (WideString streamContents: [ :stream |
		256 to: 260 do: [ :code | 
			stream nextPut: code asCharacter ] ]) asSymbol.

	self analyzer considerGlobal: aWideSymbol.

	aPerson := FLPerson new.
	Smalltalk globals at: aWideSymbol put: aPerson.
	
	self assertSerializationIdentityOf: aPerson.
! !

!FLBasicSerializationTest methodsFor: 'tests-cycles' stamp: 'MarianoMartinezPeck 9/28/2011 22:35'!
testCyclicIdentitySet

	| aSet materializedSet |
	aSet := IdentitySet new.
	aSet add: aSet.
	
	materializedSet := self resultOfSerializeAndMaterialize: aSet.
	self assert: aSet ~~ materializedSet .
	self assert: (materializedSet includes: materializedSet) description: 'The materialized set has to include himself. Note aSet = materializedSet is false, bacause equality implementation checks that materializedSet *identity-includes* each element of aSet, which is false.'! !

!FLBasicSerializationTest methodsFor: 'tests-cycles' stamp: 'MartinDias 3/21/2011 16:08'!
testCyclicLink

	| a materializedA |
	a := Link new.
	a nextLink: a.
	
	materializedA := self resultOfSerializeAndMaterialize: a.

	"We can't check using the equality of the links because hangs ad infinitum. So we only check that the structure is right."
	self assert: materializedA nextLink == materializedA! !

!FLBasicSerializationTest methodsFor: 'tests-cycles' stamp: 'MartinDias 11/11/2010 19:31'!
testCyclicLinks

	| a b c materializedA |
	a := Link new.
	b := Link new.
	c := Link new.
	
	a nextLink: b.
	b nextLink: c.
	c nextLink: a.
	
	materializedA := self resultOfSerializeAndMaterialize: a.

	"We can't check using the equality of the links because hangs ad infinitum. So we only check that the structure is right."
	self assert: materializedA nextLink nextLink nextLink == materializedA.
	self deny: materializedA nextLink == materializedA.
	self deny: materializedA nextLink nextLink == materializedA.! !

!FLBasicSerializationTest methodsFor: 'tests-cycles' stamp: 'MartinDias 10/5/2011 18:15'!
testCyclicLinksInArray

	| a b c materializedA materialized |
	a := Link new.
	b := Link new.
	c := Link new.
	
	a nextLink: b.
	b nextLink: c.
	c nextLink: a.
	
	materialized := self resultOfSerializeAndMaterialize: (Array with: a with: b with: c).
	materializedA := materialized first.

	"We can't check using the equality of the links because hangs ad infinitum. So we only check that the structure is right."
	self assert: materialized size = 3.
	self assert: materializedA nextLink nextLink nextLink == materializedA.
	self deny: materializedA nextLink == materializedA.
	self deny: materializedA nextLink nextLink == materializedA.! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 9/2/2011 10:42'!
testDate

	self assertSerializationEqualityOf: (Date fromDays: 37023).
	self assertSerializationEqualityOf: (Date today).
	self assertSerializationEqualityOf: (Date year: 3050 month: 12 day: 31).
	self assertSerializationEqualityOf: (Date year: 1600 month: 12 day: 31).
	 
	
	
	
	! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 9/2/2011 10:41'!
testDateAndTime
	
	| initialTime initialDate |
	initialTime := Time fromSeconds: 76020.
	initialDate := Date fromSeconds: 3492288000.
	self assertSerializationEqualityOf: (DateAndTime date: initialDate time: initialTime).
	
	initialTime := (Time hour: 24 minute: 60 second: 60).
	initialDate := Date year: 3050 month: 12 day: 31.
	self assertSerializationEqualityOf: (DateAndTime date: initialDate time: initialTime).
	
	initialTime := (Time hour: 24 minute: 60 second: 60).
	initialDate := Date year: 1600 month: 12 day: 31.
	self assertSerializationEqualityOf: (DateAndTime date: initialDate time: initialTime).
	! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MartinDias 10/17/2010 20:59'!
testDictionary

	self assertSerializationEqualityOf: (Dictionary with: 1->2).! !

!FLBasicSerializationTest methodsFor: 'tests-globals' stamp: 'MartinDias 1/10/2012 14:29'!
testDontConsiderCustomGlobal
	"A custom global variable is not treated as global by Fuel, unless we explicitly specify this."
	
	| aPerson |
	aPerson := FLPerson new.
	Smalltalk globals at: #FLGlobalVariableForTesting put: aPerson.
	
	self deny: (self resultOfSerializeAndMaterialize: aPerson) == aPerson. 
! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 9/9/2011 01:40'!
testDuration
	
	self assertSerializationEqualityOf: 123 seconds.
	self assertSerializationEqualityOf: -123 seconds.
	self assertSerializationEqualityOf: (
		Duration 
			seconds: 3 
			nanoSeconds: 35).
	
! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MartinDias 10/26/2010 19:46'!
testEmptyArray
	self assertSerializationEqualityOf: #()! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MartinDias 10/26/2010 19:45'!
testEmptyDictionary

	self assertSerializationEqualityOf: Dictionary new! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MartinDias 10/26/2010 19:46'!
testEmptyOrderedCollection

	self assertSerializationEqualityOf: OrderedCollection new! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MartinDias 6/5/2011 01:52'!
testEmptySet

	self assertSerializationEqualityOf: Set new! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 10/11/2011 12:06'!
testExceptions
	| ex |
	ex := (Error new messageText: 'sample error').
	self assert: (self resultOfSerializeAndMaterialize: ex) messageText equals: ex messageText.
	
	ex := (Warning new messageText: 'sample warning').
	self assert: (self resultOfSerializeAndMaterialize: ex) messageText equals: ex messageText.
	
	ex := (Notification new messageText: 'sample notification').
	self assert: (self resultOfSerializeAndMaterialize: ex) messageText equals: ex messageText.
	
! !

!FLBasicSerializationTest methodsFor: 'tests-fuelAfterMaterialization' stamp: 'MarianoMartinezPeck 4/19/2012 19:10'!
testExecuteAfterMaterialization

	| anObject result aClass |
	aClass := 
		self newClass
			addInstVarNamed: 'a';
			duringTestCompileSilently: 'fuelAfterMaterialization a := #A';
			yourself.
	anObject := aClass new.

	result := self resultOfSerializeAndMaterialize: anObject.

	self assert: #A = (result instVarNamed: 'a').
! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 20:59'!
testFalse

	self assertSerializationIdentityOf: false! !

!FLBasicSerializationTest methodsFor: 'tests-numbers' stamp: 'MarianoMartinezPeck 10/11/2011 12:07'!
testFloat
	
	self assertSerializationEqualityOf: 180.0.
	self assertSerializationEqualityOf: 0.0.
	self assertSerializationEqualityOf: -0.0.
	self assertSerializationEqualityOf: 11.22321.
	self assertSerializationEqualityOf: -11.22321.
	self assertSerializationEqualityOf: -132311.22321.
	self assertSerializationEqualityOf: 1234567890.123456789.
	self assertSerializationEqualityOf: -1234567890.123456789.
	self assertSerializationEqualityOf: Float e.
	self assertSerializationEqualityOf: Float infinity.
	self assertSerializationEqualityOf: Float halfPi.
	self assertSerializationEqualityOf: Float negativeZero.
	self assertSerializationEqualityOf: Float halfPi.
	self assertSerializationEqualityOf: Float pi.
! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 10/11/2011 12:12'!
testFraction

	self assertSerializationEqualityOf: (3 / 4).
	self assertSerializationEqualityOf: (4 / 3).
	self assertSerializationEqualityOf: (-4 / 3).
	self assertSerializationEqualityOf: (-4 / -3).
	self assertSerializationEqualityOf: (4 / -3).	
	self assertSerializationIdentityOf: (0-0 / -3).	
	self assertSerializationEqualityOf: 0-(1001/1000).	


! !

!FLBasicSerializationTest methodsFor: 'tests-globals' stamp: 'MartinDias 12/30/2011 18:44'!
testGlobalClass
	"A class should be global by default."
	
	self assertSerializationIdentityOf: Integer.
! !

!FLBasicSerializationTest methodsFor: 'tests-globals' stamp: 'MartinDias 12/30/2011 18:44'!
testGlobalMetaclass
	"A metaclass should be global by default."
	
	self assertSerializationIdentityOf: Integer class.
! !

!FLBasicSerializationTest methodsFor: 'tests-not-so-basic' stamp: 'MartinDias 8/8/2011 15:47'!
testGradientFillStyle

	self assertSerializationEqualityOf: GradientFillStyle sample! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MarianoMartinezPeck 6/6/2011 23:10'!
testInterval

	self assertSerializationEqualityOf: (-10 to: 10 by: 5).
	self assertSerializationEqualityOf: (-0 to: 0 by: 1).
	self assertSerializationEqualityOf: (1 to: 10 by: 2).
	self assertSerializationEqualityOf: (33333333333333331 to: 1444444444444444440 by: 2).
	self assertSerializationEqualityOf: (0 to: 1 by: 2).
	! !

!FLBasicSerializationTest methodsFor: 'tests-numbers' stamp: 'MarianoMartinezPeck 10/11/2011 12:12'!
testLargeNegativeInteger

	self assertSerializationEqualityOf: -354314316134313999999999.
	self assertSerializationEqualityOf: 0-100 factorial.! !

!FLBasicSerializationTest methodsFor: 'tests-numbers' stamp: 'MarianoMartinezPeck 10/11/2011 12:12'!
testLargePositiveInteger

	self assertSerializationEqualityOf: 354314316134313999999999.
	self assertSerializationEqualityOf: 100 factorial.
	! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 6/3/2011 21:20'!
testLotsOfCharacters

	| all |
	all := OrderedCollection new.
	(1 << 16) timesRepeat: [
	all add: (1 to: 255 ) atRandom asCharacter ].
	self assertSerializationEqualityOf: all.! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 6/3/2011 20:27'!
testLotsOfNils

	self assertSerializationEqualityOf: (Array new: 1 << 16).! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MarianoMartinezPeck 4/20/2012 21:37'!
testMethodDictionary
	self resultOfSerializeAndMaterializeMethodDictionary: FLPair methodDict.
	! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MarianoMartinezPeck 6/20/2011 17:11'!
testNestedDictionary

	| childDic dic |
	childDic := Dictionary new.
	childDic at: #child1 put: 'abcde'.
	dic := Dictionary new.
	dic at: #parent1 put: 'sample string.'.
	dic at: #parent2 put: 100.
	dic at: #parent3 put: #(10 20 30 ).
	dic at: #parent4 put: childDic.
	
	self assertSerializationEqualityOf: childDic.
	self assertSerializationEqualityOf: dic.! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 20:56'!
testNil

	self assertSerializationIdentityOf: nil! !

!FLBasicSerializationTest methodsFor: 'tests-not-so-basic' stamp: 'MartinDias 5/19/2012 18:48'!
testNotSerializableObject

	self 
		should: [ self serialize: FLNotSerializableMock new ] 
		raise: FLNotSerializable
		whoseDescriptionIncludes: FLNotSerializableMock new printString
		description: 'User can prevent some objects from serialization.'! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MartinDias 9/29/2010 11:27'!
testOrderedCollection

	self assertSerializationEqualityOf: (OrderedCollection with: 10 with: 20)! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 11/25/2010 18:40'!
testPair

	self assertSerializationEqualityOf:
			(FLPair new
				left: 10;
				right: 20;
				yourself)! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 9/2/2011 14:25'!
testPoint

	self assertSerializationEqualityOf: 10@20.
	self assertSerializationEqualityOf: -10@20.
	self assertSerializationEqualityOf: -10@ -20.
	self assertSerializationEqualityOf: 1.1@2.2.
	self assertSerializationEqualityOf: 1.1@ -2.2.
	! !

!FLBasicSerializationTest methodsFor: 'tests-streams' stamp: 'MartinDias 10/5/2011 18:15'!
testReadStream

	| aReadStream materialized |
	aReadStream := ReadStream on: 'A'.

	materialized := self resultOfSerializeAndMaterialize: aReadStream.
	
	self assert: $A = aReadStream next.

	self deny: materialized atEnd.
	self assert: $A = materialized next.
	self assert: materialized atEnd.! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 9/2/2011 14:25'!
testRectangle
	
	self assertSerializationEqualityOf: (10@20 corner: 30@40).
	self assertSerializationEqualityOf: (1.1@2.2 corner: 3.3@4.4).
	self assertSerializationEqualityOf: (1.1@ -2.2 corner: -3.3@4.4).! !

!FLBasicSerializationTest methodsFor: 'tests-cycles' stamp: 'MartinDias 10/5/2011 18:16'!
testRecursiveArray

	| arr materialized |
	arr := Array new: 3.
	arr at: 1 put: 10.
	arr at: 2 put: 20.
	arr at: 3 put: arr.
	
	materialized := self resultOfSerializeAndMaterialize: arr.

	"We can't check using the equality of the links because hangs ad infinitum. So we only check that the structure is right."
	self assert: materialized first = 10.
	self assert: materialized second = 20.
	self assert: materialized third == materialized.! !

!FLBasicSerializationTest methodsFor: 'tests-strings' stamp: 'MartinDias 10/5/2011 18:17'!
testRemoteString
	"Tests that the materialized RemoteString has *the same* location on file."

	| aRemoteString materializedRemoteString |	
	aRemoteString := RemoteString new setSourcePointer: thisContext method sourcePointer.

	materializedRemoteString := self resultOfSerializeAndMaterialize: aRemoteString.

	self assert: aRemoteString string = materializedRemoteString string.
	self assert: aRemoteString sourcePointer == materializedRemoteString sourcePointer.! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MarianoMartinezPeck 6/20/2011 17:21'!
testRunArray

	self assertSerializationEqualityOf: (RunArray runs: #(1 2 1) values: #(1 2 3))
! !

!FLBasicSerializationTest methodsFor: 'tests-numbers' stamp: 'MarianoMartinezPeck 10/11/2011 12:14'!
testScaledDecimal

	self assertSerializationEqualityOf: (13/11s6).
	self assertSerializationEqualityOf: (-13/11s6).
	self assertSerializationEqualityOf: (7621476292473147/9007199254740992s8).
	self assertSerializationEqualityOf: (-7621476292473147/9007199254740992s8).
	self assertSerializationEqualityOf: (-0/9007199254740992s8).
	self assertSerializationEqualityOf: ((13 / 11) asScaledDecimal: 6).
	self assertSerializationEqualityOf: ((11 / 13) asFloat asScaledDecimal).
! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MartinDias 10/5/2011 18:18'!
testSet
	| set materialized |
	set := Set new.
	set add: 10.
	set add: 20.
	set add: 30.
	set add: 30.
	
	materialized := self resultOfSerializeAndMaterialize: set.
	self assert: set = materialized.
	self assert: materialized size = 3.
	self assert: (materialized includes: 10).
	self assert: (materialized includes: 20).
	self assert: (materialized includes: 30).	! !

!FLBasicSerializationTest methodsFor: 'tests-collections-Pharo1.3' stamp: 'MartinDias 9/9/2011 12:49'!
testSetElement

	self assertSerializationEqualityOf: (SetElement with: 3)! !

!FLBasicSerializationTest methodsFor: 'tests-collections-Pharo1.3' stamp: 'MartinDias 8/13/2011 20:58'!
testSetWithNil

	self assertSerializationEqualityOf: (Set with: nil)! !

!FLBasicSerializationTest methodsFor: 'tests-collections-Pharo1.3' stamp: 'MartinDias 8/13/2011 21:00'!
testSetWithSetElement

	self assertSerializationEqualityOf: (Set with: (SetElement with: 3))! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 18:18'!
testSharedReferences
	| oneDotOne  point1 point2 materialized array |
	
	oneDotOne := 1.1.
	
	"I cannot directly use 4.2 because the Compiler reuses the same float instance 4.2 for the literals and hence the last assert fails. "
	point1:= Point x: oneDotOne y: (Float readFrom: '4.2' readStream).
	point2:= Point x: (Float readFrom: '4.2' readStream) y: oneDotOne.
	array := (Array with: point1 with: point2).
	
	materialized := self resultOfSerializeAndMaterialize: array.
	self assert: array = materialized.
	self assert: materialized first x == materialized second y.
	self deny: materialized first y == materialized second x.
	

	
	! !

!FLBasicSerializationTest methodsFor: 'tests-numbers' stamp: 'MarianoMartinezPeck 10/11/2011 12:16'!
testSmallInteger

	self assertSerializationIdentityOf: -1212.
	self assertSerializationIdentityOf: 7.
	self assertSerializationIdentityOf: 0.
	self assertSerializationIdentityOf: 1111.
	self assertSerializationIdentityOf: SmallInteger one.

! !

!FLBasicSerializationTest methodsFor: 'tests-numbers' stamp: 'MartinDias 10/17/2010 20:57'!
testSmallIntegerMaxValue

	self assertSerializationIdentityOf: SmallInteger maxVal .
	! !

!FLBasicSerializationTest methodsFor: 'tests-numbers' stamp: 'MartinDias 10/17/2010 21:04'!
testSmallIntegerMinValue

	self assertSerializationIdentityOf: SmallInteger minVal ! !

!FLBasicSerializationTest methodsFor: 'tests-numbers' stamp: 'MartinDias 9/29/2010 11:27'!
testSmallIntegerNegative

	self assertSerializationIdentityOf: -42.! !

!FLBasicSerializationTest methodsFor: 'tests-globals' stamp: 'MartinDias 1/10/2012 14:30'!
testSmalltalkGlobals
	"Smalltalk globals should be global by default."

	self assertSerializationIdentityOf: Smalltalk globals! !

!FLBasicSerializationTest methodsFor: 'tests-strings' stamp: 'MarianoMartinezPeck 10/11/2011 12:17'!
testString

	self assertSerializationEqualityOf: 'testString'.
	self assertSerializationEqualityOf:  'Hi, I''m String-object'.
	self assertSerializationEqualityOf:  String new.
	self assertSerializationEqualityOf:  (0 to: 255) asByteArray asString.
	
! !

!FLBasicSerializationTest methodsFor: 'tests-strings' stamp: 'MarianoMartinezPeck 10/11/2011 12:20'!
testSymbol

	self assertSerializationIdentityOf: #testSymbol.
	self assertSerializationIdentityOf: #with:with:with:with:.
	self assertSerializationIdentityOf: #'hello there'.! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 9/20/2011 20:39'!
testTime

	self assertSerializationEqualityOf: (Time fromSeconds: 84072).
	self assertSerializationEqualityOf: (Time hour: 24 minute: 60 second: 60).
	self assertSerializationEqualityOf: (Time hour: 23 minute: 59 second: 59).
	self assertSerializationEqualityOf: (Time hour: 0 minute: 0 second: 0).
	self assertSerializationEqualityOf: (Time seconds: 0 nanoSeconds: 5).
	self assertSerializationEqualityOf: (Time allInstances sort:  [:a :b | a asSeconds > b asSeconds]) first.
	self assertSerializationEqualityOf: (Time allInstances sort:  [:a :b | a nanoSecond > b nanoSecond]) first
		
! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 8/8/2011 15:47'!
testTimestamp

	self assertSerializationEqualityOf: (TimeStamp date: (Date fromSeconds: 3330720000) time: (Time fromSeconds: 13506) ).
	self assertSerializationEqualityOf: (Date today asTimeStamp).
	

! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/17/2010 20:59'!
testTrue

	self assertSerializationIdentityOf: true! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 10/11/2011 12:21'!
testUUID

	self assertSerializationEqualityOf: (UUID fromString: 'a3b64357-377a-5b41-b575-1c653084a121').
	self assertSerializationEqualityOf: UUID nilUUID.
	self assertSerializationEqualityOf: UUID new.
	! !

!FLBasicSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/17/2012 20:36'!
testWideAndByteCharacters
    "Since wide and byte characters are represented in the same class, this case is prone to fail."
	
	| materialized aByteCharacter aWideCharacter|
	aWideCharacter := Character value: 12345.
	aByteCharacter := $a.
	materialized := self resultOfSerializeAndMaterialize: (Array with: aWideCharacter with: aByteCharacter ).
	self assert: materialized first = aWideCharacter.
	self assert: materialized second == aByteCharacter.! !

!FLBasicSerializationTest methodsFor: 'tests-strings' stamp: 'dsfsfs 6/26/2012 09:57'!
testWideString

	self assertSerializationEqualityOf: 'aString' asWideString.
	self assertSerializationEqualityOf: (WideString
               streamContents: [ :stream |
                       2000 timesRepeat: [
                               stream nextPut: (256 to: 1000) atRandom asCharacter ] ] ).
! !

!FLBasicSerializationTest methodsFor: 'tests-collections' stamp: 'MartinDias 3/21/2011 16:10'!
testWordArray

	self assertSerializationEqualityOf: (WordArray with: 10 with: 20)! !

!FLBasicSerializationTest methodsFor: 'tests-streams' stamp: 'MartinDias 10/5/2011 18:18'!
testWriteStream

	| aWriteStream materialized |
	aWriteStream := WriteStream on: (ByteArray new: 1).

	materialized := self resultOfSerializeAndMaterialize: aWriteStream.
	
	aWriteStream nextPut: 1.
	self assert: 1 = aWriteStream size.

	self assert: 0 = materialized size.
	materialized nextPut: 2.
	self assert: (Array with: 2) = materialized contents asArray.
! !

FLBasicSerializationTest subclass: #FLInMemoryBasicSerializationTest
	instanceVariableNames: 'byteArray'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLInMemoryBasicSerializationTest class methodsFor: 'testing' stamp: 'MartinDias 3/28/2012 22:49'!
shouldInheritSelectors 
	^ true! !

!FLInMemoryBasicSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/28/2012 22:48'!
materialization

	self error: 'Disabled, instead use #materialized.'
! !

!FLInMemoryBasicSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/28/2012 22:48'!
materialized

	^ FLMaterializer materializeFromByteArray: byteArray.
! !

!FLInMemoryBasicSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/28/2012 22:46'!
serialize: anObject

	byteArray := FLSerializer serializeToByteArray: anObject! !

!FLInMemoryBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/28/2012 22:58'!
testConsiderCustomGlobal
	"Since FLSerializer class >> serializeToByteArray: does not allow customizing serialization, this test is disabled."! !

!FLInMemoryBasicSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/28/2012 22:56'!
testStringGlobalsAndClosure

	| materialized |
	materialized := self resultOfSerializeAndMaterialize:  (Array 
			with: 'a string' 
			with: Transcript
			with: [ Transcript show: 'a string' ]).
	self assert: materialized first = 'a string'.
	self assert: materialized second == Transcript! !

FLSerializationTest subclass: #FLBlockClosureSerializationTest
	instanceVariableNames: ''
	classVariableNames: 'ClassVariableForTesting'
	poolDictionaries: ''
	category: 'FuelTests'!

FLBlockClosureSerializationTest class
	instanceVariableNames: 'interval'!

FLBlockClosureSerializationTest class
	instanceVariableNames: 'interval'!

!FLBlockClosureSerializationTest class methodsFor: 'closures for testing' stamp: 'MartinDias 3/23/2012 12:08'!
blockClosureWithSelfSend
	^ [ self printString ]! !

!FLBlockClosureSerializationTest class methodsFor: 'closures for testing' stamp: 'mada 5/4/2012 11:10'!
blockClosureWithTempVariableRead
	| string |
	string := 'test'.
	^ [ string asUppercase ].! !

!FLBlockClosureSerializationTest methodsFor: 'tests-change' stamp: 'MarianoMartinezPeck 4/19/2012 19:10'!
testBlockClosureChangeDifferentBytecodes
	"Raise an error when materializing a closure whose method has changed bytecodes."

	| aClass aClosure |
	aClass := self newClass
		duringTestCompileSilently: 'methodWithClosure  ^ [ 42 ]';
		yourself.
	aClosure := aClass new perform: #methodWithClosure.
	self serialize: aClosure.
	aClass duringTestCompileSilently: 'methodWithClosure  ^ 42'.
	self should: [ self materialized ] raise: FLMethodChanged! !

!FLBlockClosureSerializationTest methodsFor: 'tests-change' stamp: 'MarianoMartinezPeck 4/19/2012 19:10'!
testBlockClosureChangeSameBytecodes
	"Tolerate materializing a closure whose method has changed but not the bytecodes."

	| aClass aClosure materializedClosure |
	aClass := self newClass
		duringTestCompileSilently: 'methodWithClosure  ^ [ 41 ]';
		yourself.
	aClosure := aClass new perform: #methodWithClosure.
	self serialize: aClosure.
	aClass duringTestCompileSilently: 'methodWithClosure  ^ [ 42 ]'.
	self deny: aClosure method isInstalled.
	materializedClosure := self materialized.
	self assert: materializedClosure value = 42! !

!FLBlockClosureSerializationTest methodsFor: 'tests-change' stamp: 'MarianoMartinezPeck 4/19/2012 19:10'!
testBlockClosureRemoved
	"Raise an error when materializing a closure whose method was removed."

	| aClass aClosure |
	aClass := self newClass
		duringTestCompileSilently: 'methodWithClosure  ^ [ 42 ]';
		yourself.
	aClosure := aClass new perform: #methodWithClosure.
	self serialize: aClosure.
	aClass removeSelectorSilently: #methodWithClosure.
	self should: [ self materialized ] raise: FLMethodNotFound! !

!FLBlockClosureSerializationTest methodsFor: 'tests-clean' stamp: 'MartinDias 3/23/2012 02:03'!
testBlockClosureWithClassVariableRead
	| closure materializedClosure |
	ClassVariableForTesting := nil.
	closure := [ ClassVariableForTesting ].
	materializedClosure := self resultOfSerializeAndMaterialize: closure.
	closure assertWellMaterializedInto: materializedClosure in: self.

	ClassVariableForTesting := true.
	self assert: materializedClosure value.
	ClassVariableForTesting := false.
	self deny: materializedClosure value! !

!FLBlockClosureSerializationTest methodsFor: 'tests-unclean' stamp: 'MartinDias 3/23/2012 12:09'!
testBlockClosureWithSelfSend
	| closure materializedClosure |
	closure := self class blockClosureWithSelfSend.
	materializedClosure := self resultOfSerializeAndMaterialize: closure.
	closure assertWellMaterializedInto: materializedClosure in: self.
	self assert: materializedClosure value = closure value! !

!FLBlockClosureSerializationTest methodsFor: 'tests-clean' stamp: 'mada 5/4/2012 11:10'!
testBlockClosureWithTempVariableRead
	| closure materializedClosure |
	closure := self class blockClosureWithTempVariableRead.
	materializedClosure := self resultOfSerializeAndMaterialize: closure.
	closure assertWellMaterializedInto: materializedClosure in: self.
	self assert: materializedClosure value = 'TEST'.
! !

!FLBlockClosureSerializationTest methodsFor: 'tests-clean' stamp: 'MartinDias 3/23/2012 02:01'!
testBlockClosureWithThreeArguments
	| closure materializedClosure |
	closure := [ :a :b :c | a + b + c ].
	materializedClosure := self resultOfSerializeAndMaterialize: closure.
	closure assertWellMaterializedInto: materializedClosure in: self.
	self assert: (materializedClosure value: 1 value: 2 value: 3) = 6! !

!FLBlockClosureSerializationTest methodsFor: 'tests-clean' stamp: 'MartinDias 3/26/2012 14:43'!
testCleanCachedBlockClosure
	| closure materializedClosure |
	closure := [ 42 ] becomeCached.
	materializedClosure := self resultOfSerializeAndMaterialize: closure.
	closure assertWellMaterializedInto: materializedClosure in: self.
	self assert: materializedClosure once = 42.
	self assert: materializedClosure value = 42! !

!FLBlockClosureSerializationTest methodsFor: 'tests-clean' stamp: 'MartinDias 3/23/2012 12:21'!
testNestedBlockClosure
	| closure materializedClosure |
	closure := [ [ 42 ] ].
	materializedClosure := self resultOfSerializeAndMaterialize: closure.
	closure assertWellMaterializedInto: materializedClosure in: self.
	self assert: materializedClosure value value = 42! !

!FLBlockClosureSerializationTest methodsFor: 'tests-unclean' stamp: 'MartinDias 3/26/2012 19:24'!
testUncleanCachedBlockClosure
	| closure materializedClosure |
	closure := self class blockClosureWithSelfSend becomeCached.
	materializedClosure := self resultOfSerializeAndMaterialize: closure.
	closure assertWellMaterializedInto: materializedClosure in: self.
	self assert: materializedClosure once = closure once.
	self assert: materializedClosure value = closure value! !

FLSerializationTest subclass: #FLClassSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLClassSerializationTest commentStamp: '<historical>' prior: 0!
I have the common behavior for testing class serialization.!

!FLClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 5/19/2012 13:42'!
during: aBlock rename: aClass as: anotherClass

	[ Smalltalk at: anotherClass name put: aClass.
	^aBlock ] ensure: [ 
	Smalltalk at: anotherClass name put: anotherClass ].
	! !

!FLClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/8/2011 03:11'!
newClassOrTrait
	"Returns a class for testing"

	^ self newClassWithInstanceVariableNames: ''! !

!FLClassSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 12/13/2011 20:45'!
newClassOrTraitWithSuperClass: superclass
	"Returns a class for testing"

	^ self newClassWithInstanceVariableNames: '' superclass: superclass! !

!FLClassSerializationTest methodsFor: 'running' stamp: 'MarianoMartinezPeck 11/21/2011 20:20'!
newClassWithInstanceVariableNames: instanceVariableNames
	"Returns a class for testing, with the specified instance variables."

	^ self newClassWithInstanceVariableNames: instanceVariableNames superclass: Object! !

!FLClassSerializationTest methodsFor: 'running' stamp: 'MarianoMartinezPeck 4/19/2012 19:10'!
newClassWithInstanceVariableNames: instanceVariableNames superclass: aSuperclass
	"Returns a class for testing, with the specified instance variables."

	^ self 
		newSubclassOf: aSuperclass
		instanceVariableNames: instanceVariableNames
		classVariableNames: ''! !

!FLClassSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/23/2011 17:42'!
newInstanceFrom: aClass

	^ aClass new! !

!FLClassSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/8/2011 04:14'!
newSecondaryTrait
	"Returns a trait for testing"

	^ self newTraitSuffixed: 'Secondary'! !

FLClassSerializationTest subclass: #FLGlobalClassSerializationTest
	uses: FLTGlobalClassOrTraitSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLGlobalClassSerializationTest commentStamp: 'MartinDias 11/28/2011 10:48' prior: 0!
I test the serialization of classes as *global* objects, i.e. the classes has to be present in the image at materialization time.!

FLGlobalClassSerializationTest class
	uses: FLTGlobalClassOrTraitSerializationTest classTrait
	instanceVariableNames: ''!

!FLGlobalClassSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 19:40'!
testInstanceOfAnObsoleteClass
	"Tests that serializer does not serialize an instance of an obsolete class."
	
	| aClass instance |
	aClass := self newClassOrTrait.
	instance := self newInstanceFrom: aClass.
	self removeFromSystem: aClass.

	self 
		should: [self serialize: instance ] 
		raise: FLObsolete
		whoseDescriptionIncludes: aClass name
		description: 'Should raise an error when serializing as global an obsolete class.' ! !

FLClassSerializationTest subclass: #FLIgnoredVariablesTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLIgnoredVariablesTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:46'!
testAllVariablesIgnored

	| anObject result aClass |
	aClass := self newClassWithInstanceVariableNames: 'a b'.
	aClass class duringTestCompileSilently: 'fuelIgnoredInstanceVariableNames ^#(a b)'.
	anObject := aClass new
		instVarAt: 1 put: $A;
		instVarAt: 2 put: $B;
		yourself.

	result := self resultOfSerializeAndMaterialize: anObject.

	self assert: (result instVarAt: 1) isNil.
	self assert: (result instVarAt: 2) isNil.
! !

!FLIgnoredVariablesTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:07'!
testIgnoredValueIsNotMaterialized

	| anObject materializedInstances aClass |
	aClass := self newClassWithInstanceVariableNames: 'a'.
	aClass class duringTestCompileSilently: 'fuelIgnoredInstanceVariableNames ^#(a)'.
	anObject := aClass new
		instVarAt: 1 put: #A;
		yourself.

	self serialize: anObject.
	materializedInstances := self materializedObjects.

	self deny: (materializedInstances includes: #A)
! !

!FLIgnoredVariablesTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:46'!
testOneIgnoredVariable

	| anObject result aClass |
	aClass := self newClassWithInstanceVariableNames: 'a b c'.
	aClass class duringTestCompileSilently: 'fuelIgnoredInstanceVariableNames ^#(b)'.
	anObject := aClass new
		instVarAt: 1 put: $A;
		instVarAt: 2 put: $B;
		instVarAt: 3 put: $C;
		yourself.

	result := self resultOfSerializeAndMaterialize: anObject.

	self assert: $A = (result instVarAt: 1).
	self assert: nil = (result instVarAt: 2).
	self assert: $C = (result instVarAt: 3)! !

!FLIgnoredVariablesTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:46'!
testTwoIgnoredVariables

	| anObject result aClass |
	aClass := self newClassWithInstanceVariableNames: 'a b c'.
	aClass class duringTestCompileSilently: 'fuelIgnoredInstanceVariableNames ^#(a c)'.
	anObject := aClass new
		instVarAt: 1 put: $A;
		instVarAt: 2 put: $B;
		instVarAt: 3 put: $C;
		yourself.

	result := self resultOfSerializeAndMaterialize: anObject.

	self assert: nil = (result instVarAt: 1).
	self assert: $B = (result instVarAt: 2).
	self assert: nil = (result instVarAt: 3).! !

FLClassSerializationTest subclass: #FLMigrationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLMigrationTest methodsFor: 'running' stamp: 'MarianoMartinezPeck 7/10/2012 16:47'!
redefined: aClass with: instanceVariableNames

	| name |
	name := aClass name.
	self removeFromSystem: aClass.
	"In this case we need to create a class with notifications"
	^ (self withNotificationsNewClassWithInstanceVariableNames: instanceVariableNames) renameSilently: name.
	! !

!FLMigrationTest methodsFor: 'tests-manual' stamp: 'MartinDias 1/6/2012 15:46'!
testBadDestinationVariableRename
	
	| pointClass aPoint pointClassName |
	pointClass := self newClassWithInstanceVariableNames: 'x y'.
	pointClassName := pointClass name.
	aPoint := pointClass new.
	
	self materializer 
		migrateClassNamed: pointClassName 
		variables: {('x' -> 'posX')}.
		
	self serialize: aPoint.
	self should: [ self materialized ] raise: Error. 
	
	self flag: #todo."Assert an specific materialization error"! !

!FLMigrationTest methodsFor: 'tests-automatic' stamp: 'MartinDias 1/6/2012 15:46'!
testChangeInSuperclass
	"Tests that serializer tolarates when there is a change in the superclass between serialization and materialization"
	
	|  aClass aClassSubclass instance materializedInstance |	
	
	aClass := self newClassWithInstanceVariableNames: 'a b c' superclass: Object.
	aClassSubclass := self newClassWithInstanceVariableNames: 'd e' superclass: aClass.
	
	instance := aClassSubclass new.
	instance instVarNamed: 'a' put: $A.	
	instance instVarNamed: 'b' put: $B.	
	instance instVarNamed: 'c' put: $C.	
	instance instVarNamed: 'd' put: $D.	
	instance instVarNamed: 'e' put: $E.	
	
	self serialize: instance.
	aClass removeInstVarNamed: 'a'.
	aClass removeInstVarNamed: 'b'.
	aClass addInstVarNamed: 'x'.
	aClassSubclass superclass: aClass.
	
	materializedInstance := self materialized.

	self assert: $D = (materializedInstance instVarNamed: 'd').
	self assert: $E = (materializedInstance instVarNamed: 'e').
	self should: [materializedInstance instVarNamed: 'a'] raise: Error. 
	self should: [materializedInstance instVarNamed: 'b'] raise: Error. 
	self assert: $C = (materializedInstance instVarNamed: 'c').
	self assert: (materializedInstance instVarNamed: 'x') isNil.
	  
	
	! !

!FLMigrationTest methodsFor: 'tests-manual' stamp: 'MartinDias 5/18/2012 23:00'!
testClassAndVariableRename
	
	| pointClass aPoint resultPoint pointClassName |
	pointClass := self newClassWithInstanceVariableNames: 'x y'.
	pointClassName := pointClass name.
	aPoint := pointClass new.
	aPoint instVarNamed: 'x' put: 7.	
	aPoint instVarNamed: 'y' put: 11.
	
	self serialize: aPoint.
	pointClass renameSilently: (pointClassName, 'Renamed') asSymbol.
	pointClass := self redefined: pointClass with: 'posY posX'.	
		
	self materializer 
		migrateClassNamed: pointClassName 
		toClass: pointClass
		variables: {'x' -> 'posX'. 'y' -> 'posY'}.
		
	resultPoint := self materialized.

	self assert: (resultPoint instVarNamed: 'posX') = 7.
	self assert: (resultPoint instVarNamed: 'posY') = 11.! !

!FLMigrationTest methodsFor: 'tests-manual' stamp: 'MarianoMartinezPeck 4/19/2012 18:01'!
testClassRename
	
	| pointClass aPoint resultPoint pointClassName |
	pointClass := self newClassWithInstanceVariableNames: 'x y'.
	pointClassName := pointClass name.
	aPoint := pointClass new.
	aPoint instVarNamed: 'x' put: 7.	
	aPoint instVarNamed: 'y' put: 11.
	
	self serialize: aPoint.
	pointClass renameSilently: (pointClassName, 'Renamed') asSymbol.

	self materializer 
		migrateClassNamed: pointClassName 
		toClass: pointClass.
		
	resultPoint := self materialized.

	self assert: (resultPoint instVarNamed: 'x') = 7.
	self assert: (resultPoint instVarNamed: 'y') = 11.! !

!FLMigrationTest methodsFor: 'tests-automatic' stamp: 'MartinDias 5/19/2012 18:39'!
testFormatFixedToVariable
	"Tests that serializer can tolerate a format change from a *fixed* class to *weak/variable*"
	
	| materialized |
	self serialize: FLPair new.
	materialized := self during: [ self materialized ] rename: FLPair as: FLWeakClassMock.
	self assert: materialized size = 0! !

!FLMigrationTest methodsFor: 'tests-automatic' stamp: 'MartinDias 5/19/2012 18:38'!
testFormatVariableToFixed
	"Tests that serializer can tolerate a format change from a *weak/variable* class to a *fixed*."
	
	| materialized |
	self serialize: (FLWeakClassMock new: 5).
	materialized := self during: [ self materialized ] rename: FLWeakClassMock as: FLPair.
	self assert: materialized size = 0! !

!FLMigrationTest methodsFor: 'tests-automatic' stamp: 'MartinDias 1/6/2012 15:46'!
testSuperclassChange
	"Tests that serializer tolarates when the superclass changed between serialization and materialization"
	
	|  aClass aClassSubclass instance materializedInstance anotherSuperclass |	
	
	aClass := self newClassWithInstanceVariableNames: 'a b c' superclass: Object.
	aClassSubclass := self newClassWithInstanceVariableNames: 'd e' superclass: aClass.
	anotherSuperclass := self newClassWithInstanceVariableNames: 'x' superclass: Object.
	
	instance := aClassSubclass new.
	instance instVarNamed: 'a' put: $A.	
	instance instVarNamed: 'b' put: $B.	
	instance instVarNamed: 'c' put: $C.	
	instance instVarNamed: 'd' put: $D.	
	instance instVarNamed: 'e' put: $E.	
	
	self serialize: instance.
	aClassSubclass superclass: anotherSuperclass.
	materializedInstance := self materialized.

	self assert: $D = (materializedInstance instVarNamed: 'd').
	self assert: $E = (materializedInstance instVarNamed: 'e').
	self should: [materializedInstance instVarNamed: 'a'] raise: Error. 
	self should: [materializedInstance instVarNamed: 'b'] raise: Error. 
	self should: [materializedInstance instVarNamed: 'c'] raise: Error. 
	self assert: (materializedInstance instVarNamed: 'x') isNil.
	  
	
	! !

!FLMigrationTest methodsFor: 'tests-automatic' stamp: 'MartinDias 1/6/2012 15:46'!
testVariableInsertion
	"Tests that serializer tolarates when there is a new instance variable on materialization"
	
	| pairClass aPair resultPair |	
	pairClass := self newClassWithInstanceVariableNames: 'left right'.
	aPair := pairClass new.
	aPair instVarAt: 1 put: $A.	
	aPair instVarAt: 2 put: $B.
	
	self serialize: aPair.
	self redefined: pairClass with: 'left middle right'.
	resultPair := self materialized.

	self assert: $A = (resultPair instVarAt: 1).
	self assert: nil = (resultPair instVarAt: 2).
	self assert: $B = (resultPair instVarAt: 3).! !

!FLMigrationTest methodsFor: 'tests-automatic' stamp: 'MartinDias 1/6/2012 15:46'!
testVariableOrderChange
	"Tests that serializer tolarates when the order in the instance variables changed between serialization and materialization"
	
	| pairClass aPair resultPair |	
	pairClass := self newClassWithInstanceVariableNames: 'left right'.
	aPair := pairClass new.
	aPair instVarAt: 1 put: $A.	
	aPair instVarAt: 2 put: $B.	
	
	self serialize: aPair.
	self redefined: pairClass with: 'right left'.
	resultPair := self materialized.

	self assert: $B = (resultPair instVarAt: 1).
	self assert: $A = (resultPair instVarAt: 2).	! !

!FLMigrationTest methodsFor: 'tests-automatic' stamp: 'MartinDias 1/6/2012 15:46'!
testVariableRemoved
	"Tests that serializer tolarates when an instance variable is missing on materialization"
	
	| pairClass aPair resultPair |	
	pairClass := self newClassWithInstanceVariableNames: 'left right'.
	aPair := pairClass new.
	aPair instVarAt: 1 put: $A.	
	aPair instVarAt: 2 put: $B.	

	self serialize: aPair.
	self redefined: pairClass with: 'right'.
	resultPair := self materialized.
	
	self assert: $B = (resultPair instVarAt: 1).	! !

!FLMigrationTest methodsFor: 'tests-manual' stamp: 'MartinDias 5/18/2012 22:59'!
testVariableRename
	
	| pointClass aPoint resultPoint pointClassName |
	pointClass := self newClassWithInstanceVariableNames: 'x y'.
	pointClassName := pointClass name.
	aPoint := pointClass new.
	aPoint instVarNamed: 'x' put: 7.	
	aPoint instVarNamed: 'y' put: 11.
	
	self serialize: aPoint.
	self redefined: pointClass with: 'posY posX'.	
		
	self materializer 
		migrateClassNamed: pointClassName 
		variables: {'x' -> 'posX'. 'y' -> 'posY'}.
		
	resultPoint := self materialized.

	self assert: (resultPoint instVarNamed: 'posX') = 7.
	self assert: (resultPoint instVarNamed: 'posY') = 11.! !

FLSerializationTest subclass: #FLGlobalSendSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLGlobalSendSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/6/2011 20:46'!
testBasic

	| result |
	self deny: FLGlobalSendMock newInstanceToSerialize = FLGlobalSendMock newInstanceToMaterialize.

	result := self resultOfSerializeAndMaterialize: FLGlobalSendMock newInstanceToSerialize.
	self assert: FLGlobalSendMock newInstanceToMaterialize = result.! !

!FLGlobalSendSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/20/2012 14:11'!
testGlobalNotFound

	self should: [ self resultOfSerializeAndMaterialize: FLGlobalSendNotPresentMock new ]
		raise: FLGlobalNotFound
		whoseDescriptionIncludes: #FLKeyNotPresentInSmalltalks 
		description: 'Should raise an error when global name is not present in Smalltalks globals.' 
	
! !

FLSerializationTest subclass: #FLHashedCollectionSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLHashedCollectionSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 19:23'!
testDictionaryRehash
	|  aDictionary person1 person2 person3 materializedDict newPerson1 |

	aDictionary := Dictionary new.
	person1 := FLPerson new id: 1.
	person2 := FLPerson new id: 5.
	person3 := FLPerson new id: 8.
	aDictionary
		at: person1 put: person1;
		at: person2 put: person2;
		at: person3 put: person3.

	self assert: (aDictionary at: person1) = person1.
	self assert: (aDictionary at: person2) = person2.
	self assert: (aDictionary at: person3) = person3.
	

	materializedDict := self resultOfSerializeAndMaterialize: aDictionary.

    "If Fuel serializes the Dictionary  normally, it will copy its internal array just as it is. Hence, the element will be in the same order. But the objects instead may be recreated. SqueakVM assigns an identity hash at creation time (stored in the object header). So, all objects who didn't implement #hash will be finally using #identityHash. I cannot change the identityHash of an object,  so I do a test changing the id by hand.
Now, let's say that the id of person 1 change to 3 (this would be the same case when Fuel creates new objects and a new identity hash is asigned) . This 3 is not coincidence. It has to be a hash such that:
    (hash \\ aDictionary array size) + 1   gives you a position in the array that has a next nil in the array. In this case, the person1 is at position 2 of the array and there is a nil in position 3 and one at 5..   So (3 \\ 5) + 1 = 4.  So....Dictionary >> scanFor:  will start to search from position 4 and since it will find a nil in 5 it will assume it has to stop searching and that the object is not present. For more details check .Dictionary  >> scanFor: "

	newPerson1 := materializedDict keys detect: [:each | each id = person1 id].
	newPerson1 id: 3.
"Fuel rehashes the set at the end. This means, after having materialized the instnaces of the set. In this case, we are chanign the has of the object once it was already been materialzed, so we should simulate the rehash done by Fuel"
	materializedDict rehash.
	self assert: (materializedDict includesKey: newPerson1).! !

!FLHashedCollectionSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 5/20/2011 13:46'!
testSetRehash
	|  aSet person1 person2 person3 materializedSet newPerson1 |
	aSet := Set new.
	person1 := FLPerson new id: 1.
	person2 := FLPerson new id: 5.
	person3 := FLPerson new id: 8.
	aSet
		add: person1;
		add: person2;
		add: person3.

	self assert: (aSet includes: person1).
	self assert: (aSet includes: person2).
	self assert: (aSet includes: person3).
	
	materializedSet := self resultOfSerializeAndMaterialize: aSet.

    "If Fuel serializes the Set normally, it will copy its internal array just as it is. Hence, the element will be in the same order. But the objects instead may be recreated. SqueakVM assigns an identity hash at creation time (stored in the object header). So, all objects who didn't implement #hash will be finally using #identityHash. I cannot change the identityHash of an object,  so I do a test changing the id by hand.
Now, let's say that the id of person 1 change to 3 (this would be the same case when Fuel creates new objects and a new identity hash is asigned) . This 3 is not coincidence. It has to be a hash such that:
    (hash \\ aSet array size) + 1   gives you a position in the array that has a next nil in the array. In this case, the person1 is at position 2 of the array and there is a nil in position 3 and one at 5..   So (3 \\ 5) + 1 = 4.  So....Set >> scanFor:  will start to search from position 4 and since it will find a nil in 5 it will assume it has to stop searching and that the object is not present. For more details check .Set >> scanFor: "
	newPerson1 := materializedSet detect: [:each | each id = person1 id].
	newPerson1 id: 3.
	"Fuel rehashes the set at the end. This means, after having materialized the instnaces of the set. In this case, we are chanign the has of the object once it was already been materialzed, so we should simulate the rehash done by Fuel"
	materializedSet rehash.
	self assert: (materializedSet includes: newPerson1).! !

FLSerializationTest subclass: #FLHeaderSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLHeaderSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 7/28/2012 13:47'!
testAdditionalObjects

	| materialization |
	self serializer at: #test putAdditionalObject: 'test'.
	self serializer at: 42 putAdditionalObject: 68.
	
	self assertSerializationEqualityOf: 'foo'.

	self assert: (self materialization additionalObjectAt: #test) equals: 'test'.
	self assert: (self materialization additionalObjectAt: 42) equals: 68.
	! !

!FLHeaderSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 7/28/2012 15:04'!
testJustMaterializeHeader

	self serializer at: #test putAdditionalObject: 'test'.
	self serializer at: 42 putAdditionalObject: 68.
	
	self serialize: 'foo'.
	
	self assert: (self materializationHeader additionalObjectAt: #test) equals: 'test'.
	self assert: (self materializationHeader additionalObjectAt: 42) equals: 68.
	! !

!FLHeaderSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 7/28/2012 12:33'!
testPostMaterializationActions

	| aClass |
	aClass := self newSubclassOf: Object instanceVariableNames: '' classVariableNames: 'ClassVariable ClassVariable2'.
	aClass class
		duringTestCompileSilently: 'postLoadMethod  ClassVariable := 1';
		duringTestCompileSilently: 'postLoadMethod2  ClassVariable := 2';
		duringTestCompileSilently: 'classVariable  ^ClassVariable '.
	
	aClass perform: #postLoadMethod.
	self assert: (aClass perform: #classVariable) = 1.
	
	self serializer addPostMaterializationAction: [  (Smalltalk at: #ClassForTestToBeDeleted1) perform: #postLoadMethod2  ].
	self serialize: aClass.
	self assert: (aClass perform: #classVariable) = 1.
	self materialized.	
	self assert: (aClass perform: #classVariable) = 2.
! !

!FLHeaderSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 7/28/2012 12:33'!
testPreMaterializationActions

	| aClass |
	aClass := self newSubclassOf: Object instanceVariableNames: '' classVariableNames: 'ClassVariable ClassVariable2'.
	aClass class
		duringTestCompileSilently: 'postLoadMethod  ClassVariable := 1';
		duringTestCompileSilently: 'postLoadMethod2  ClassVariable := 2';
		duringTestCompileSilently: 'classVariable  ^ClassVariable '.
	
	aClass perform: #postLoadMethod.
	self assert: (aClass perform: #classVariable) = 1.
	
	self serializer addPreMaterializationAction: [  (Smalltalk at: #ClassForTestToBeDeleted1) perform: #postLoadMethod2  ].
	self serialize: aClass.
	self assert: (aClass perform: #classVariable) = 1.
	self materialized.	
	self assert: (aClass perform: #classVariable) = 2.
! !

FLSerializationTest subclass: #FLHookedSubstitutionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLHookedSubstitutionTest methodsFor: 'tests' stamp: 'MartinDias 5/9/2012 01:00'!
testAvoidRecursion

	| aClass result original | 
	original := FLClassWithRecursiveSubstitution new index: 1.
	
	result := self resultOfSerializeAndMaterialize: original.
	
	self assert: result index = 2! !

!FLHookedSubstitutionTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 19:12'!
testClassWithCachedValueByNil

	| aClassWithCachedValue result original | 
	aClassWithCachedValue := (self newSubclassOf: Object instanceVariableNames: 'cache' classVariableNames: '') 
		duringTestCompileSilently: 'cache ^cache';
		duringTestCompileSilently: 'cache: x cache := x';
		duringTestCompileSilently: '
			fuelAccept: aVisitor  
				^cache 
					ifNil: [ super fuelAccept: aVisitor ]
					ifNotNil: [ aVisitor visitSubstitution: self by: (self copy cache: nil)]';
		yourself.

	original := aClassWithCachedValue new cache: 5; yourself.

	result := self resultOfSerializeAndMaterialize: original.
	
	self assert: result cache isNil.! !

!FLHookedSubstitutionTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 19:12'!
testObjectByProxyThatBecomesItsContent
	"Tests a substitution of an object by a proxy that becomes another object on materialization."

	| aProxyClass result | 
	aProxyClass := (self newSubclassOf: Object instanceVariableNames: 'someState' classVariableNames: '') 
		duringTestCompileSilently: 'initialize  someState := 5@1';
		duringTestCompileSilently: 'fuelAccept: aVisitor  ^aVisitor visitSubstitution: self by: (FLProxyThatBecomesItsContent newWith: someState)';
		yourself.

	result := self resultOfSerializeAndMaterialize: aProxyClass new.
	
	self assert: 5@1 = result.! !

!FLHookedSubstitutionTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 19:12'!
testProxyByTarget

	| aProxyClass result original | 
	aProxyClass := (self newSubclassOf: Object instanceVariableNames: 'target' classVariableNames: '') 
		duringTestCompileSilently: 'target: x  target := x';
		duringTestCompileSilently: 'fuelAccept: aVisitor  ^aVisitor visitSubstitution: self by: target';
		yourself.

	original := aProxyClass new target: 5; yourself.

	result := self resultOfSerializeAndMaterialize: original.
	
	self assert: 5 = result.! !

!FLHookedSubstitutionTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 19:12'!
testProxyByTargetAnalisysIsPropagated

	| aProxyClass result pair | 
	self flag: #todo. "I don't understand what are we testing here."

	aProxyClass := (self newSubclassOf: Object instanceVariableNames: 'target' classVariableNames: '') 
		duringTestCompileSilently: 'target: x  target := x';
		duringTestCompileSilently: 'fuelAccept: aVisitor  ^aVisitor visitSubstitution: self by: target';
		yourself.

	pair := FLPair new.
	pair left: 6.
	pair right: 'foo'. 

	aProxyClass new target: pair; yourself.

	result := self resultOfSerializeAndMaterialize: pair.
	
	self assert: result left = 6.
	self assert: result right = 'foo'.! !

!FLHookedSubstitutionTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 19:12'!
testProxyByTargetInsideObjectAndAnalisysIsPropagated

	| aProxyClass result original pair pairRoot | 
	aProxyClass := (self newSubclassOf: Object instanceVariableNames: 'target' classVariableNames: '') 
		duringTestCompileSilently: 'target: x  target := x';
		duringTestCompileSilently: 'fuelAccept: aVisitor  ^aVisitor visitSubstitution: self by: target';
		yourself.

	pair := FLPair new.
	pair left: 6.
	pair right: 'foo'. 

	original := aProxyClass new target: pair; yourself.

	pairRoot := FLPair new left: original; yourself.

	result := self resultOfSerializeAndMaterialize: pairRoot.
	
	self assert: result left left = 6.
	self assert: result left right = 'foo'.
	self assert: result right isNil.
			
! !

!FLHookedSubstitutionTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 19:12'!
testProxyInsideObjectByTarget

	| aProxyClass result original pair | 
	aProxyClass := (self newSubclassOf: Object instanceVariableNames: 'target' classVariableNames: '') 
		duringTestCompileSilently: 'target: x  target := x';
		duringTestCompileSilently: 'fuelAccept: aVisitor  ^aVisitor visitSubstitution: self by: target';
		yourself.

	original := aProxyClass new target: 5; yourself.
	
	pair := FLPair new.
	pair left: original.
	pair right: 'foo'. 

	result := self resultOfSerializeAndMaterialize: pair.
	
	self assert: result left = 5.
	self assert: result right = 'foo'.
		
! !

!FLHookedSubstitutionTest methodsFor: 'tests' stamp: 'MartinDias 5/9/2012 01:17'!
testSubstituteByItself

	| aRecursiveClass result | 
	aRecursiveClass := (self newSubclassOf: Object instanceVariableNames: '' classVariableNames: '') 
		duringTestCompileSilently: 'fuelAccept: aMapper  
			^aMapper 
				visitSubstitution: self by: self 
				onRecursionDo: [super fuelAccept: aMapper]';
		yourself.

	result := self resultOfSerializeAndMaterialize: aRecursiveClass new.	
	
	"self flag: #todo." "Should test that it doesn't enter in loop."! !

!FLHookedSubstitutionTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 19:12'!
testTransientByNil

	| result aClassOfTransientObjects | 
	aClassOfTransientObjects := self newClass
		duringTestCompileSilently: 'fuelAccept: aVisitor  ^aVisitor visitSubstitution: self by: nil';
		yourself.

	result := self resultOfSerializeAndMaterialize: aClassOfTransientObjects new.
	
	self assert: result isNil.! !

FLSerializationTest subclass: #FLMethodContextSerializationTest
	instanceVariableNames: 'thisContextSample'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLMethodContextSerializationTest class methodsFor: 'closures for testing' stamp: 'MarianoMartinezPeck 5/24/2011 13:44'!
blockClosureContextTesting
	^ [self class] asContext! !

!FLMethodContextSerializationTest class methodsFor: 'closures for testing' stamp: 'MarianoMartinezPeck 5/24/2011 13:44'!
blockClosureContextTestingWithSender: aSenderContext
	^ [self class] asContextWithSender: aSenderContext! !

!FLMethodContextSerializationTest class methodsFor: 'closures for testing' stamp: 'MarianoMartinezPeck 7/1/2011 11:11'!
blockClosureContextWithOutPointerTesting
	| string |
	string := 'test'.
	^ [self class. string asUppercase] asContext! !

!FLMethodContextSerializationTest class methodsFor: 'running' stamp: 'MartinDias 3/26/2012 22:54'!
thisContextSample
	^ thisContext copy! !

!FLMethodContextSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/26/2012 22:52'!
setUp
	super setUp.
	thisContextSample := self class thisContextSample.! !

!FLMethodContextSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/28/2012 01:19'!
testFuelShouldIgnoreFuel
	"When serializing contexts, it may happen that internal objects used by Fuel such as clusters, mappers, etc., as found in the temps of the contexts, and hence they are serialized. This is something not desired. With this test we make sure that when we serialize a whole context, there is no Fuel objects. "
	
	| context serialization fuelClasses |
	3 timesRepeat: [Smalltalk garbageCollect].
	context := thisContextSample.
	fuelClasses := (PackageInfo named: 'Fuel') classes.
	serialization := self serializationOf: context.	
	serialization clusters do: [:cluster | 
		cluster objects do: [:object |
			self deny: (fuelClasses includes: object class) ] ].
! !

!FLMethodContextSerializationTest methodsFor: 'tests' stamp: 'MartinDias 12/3/2011 22:21'!
testMethodContext
	"This test should be improved"
	| methodContext1  materializedMethodContext1  |
	methodContext1 := (MethodContext newForMethod: FLPair >> #method1).
	methodContext1 
		initializeWith: 23 
		stackPtr: 1 
		method: FLPair >> #method1 
		receiver:  (FLPair new right: 4; left: 2) 
		sender: nil.
		
	materializedMethodContext1 := self resultOfSerializeAndMaterialize: methodContext1.
	methodContext1 assertWellMaterializedInto: materializedMethodContext1 in: self.! !

!FLMethodContextSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/26/2012 22:52'!
testMethodContextThisContext

	| materializedContext context |
	context := thisContextSample.
	materializedContext := self resultOfSerializeAndMaterialize: context.
	
	self deny:  context == materializedContext.
	"I cannot compare by = because MethodContext do not implement it and it will finally use the identity."
	self assert: context pc = materializedContext pc.
	self assert: context stackPtr = materializedContext stackPtr.
	self assert: context method = materializedContext method.
	self assert: context receiver = materializedContext receiver.
	"I cannot compare by = because MethodContext do not implement it and it will finally use the identity."
	self deny: context sender ==  materializedContext sender.
	self assert: context sender pc == materializedContext sender pc.
	self assert: context sender stackPtr = materializedContext sender stackPtr.
	"etc...."
! !

!FLMethodContextSerializationTest methodsFor: 'tests' stamp: 'MartinDias 12/3/2011 22:21'!
testMethodContextWithClosure
	"This test should be improved"
	| methodContext1  materializedMethodContext1  |
	methodContext1 := self class blockClosureContextTesting.
	materializedMethodContext1 := self resultOfSerializeAndMaterialize: methodContext1.
	methodContext1 assertWellMaterializedInto: materializedMethodContext1 in: self.
	
! !

!FLMethodContextSerializationTest methodsFor: 'tests' stamp: 'MartinDias 12/3/2011 22:22'!
testMethodContextWithClosureAndSender
	"This test should be improved"
	| methodContext1  materializedMethodContext2  methodContext2 |
	
	methodContext1 := (MethodContext newForMethod: FLPair >> #method1).
	methodContext1 
		initializeWith: 23 
		stackPtr: 1 
		method: FLPair >> #method1 
		receiver:  (FLPair new right: 4; left: 2) 
		sender: nil.
	
	methodContext2 := self class blockClosureContextTestingWithSender: methodContext1.
	materializedMethodContext2 := self resultOfSerializeAndMaterialize: methodContext2.
	methodContext2 assertWellMaterializedInto: materializedMethodContext2 in: self.
	
! !

!FLMethodContextSerializationTest methodsFor: 'tests' stamp: 'MartinDias 12/3/2011 22:22'!
testMethodContextWithClosureContextWithOutPointerTesting
	"This test should be improved"
	| methodContext1  materializedMethodContext1  |
	methodContext1 := self class blockClosureContextWithOutPointerTesting.
	self assert: (methodContext1 tempNamed: 'string') = 'test'. 
	materializedMethodContext1 := self resultOfSerializeAndMaterialize: methodContext1.
	self assert: (materializedMethodContext1 tempNamed: 'string') = 'test'. 
	methodContext1 assertWellMaterializedInto: materializedMethodContext1 in: self.
	
! !

!FLMethodContextSerializationTest methodsFor: 'tests' stamp: 'MartinDias 12/3/2011 22:22'!
testMethodContextWithNilPc
	"This test should be improved"
	| methodContext1  materializedMethodContext1  |
	methodContext1 := (MethodContext newForMethod: FLPair >> #method1).
	methodContext1 
		initializeWith: nil 
		stackPtr: 1 
		method: FLPair >> #method1 
		receiver:  (FLPair new right: 4; left: 2) 
		sender: nil.
		
	materializedMethodContext1 := self resultOfSerializeAndMaterialize: methodContext1.
	methodContext1 assertWellMaterializedInto: materializedMethodContext1 in: self.! !

!FLMethodContextSerializationTest methodsFor: 'tests' stamp: 'MartinDias 12/3/2011 22:22'!
testMethodContextWithSender
	"This test should be improved"
	| methodContext1 methodContext2 materializedMethodContext1 |
	methodContext1 := (MethodContext newForMethod: FLPair >> #method1).
	methodContext1 
		initializeWith: 23 
		stackPtr: 1 
		method: FLPair >> #method1 
		receiver:  (FLPair new right: 4; left: 2) 
		sender: nil.
		
	methodContext2 := (MethodContext newForMethod: FLPair >> #method2).
	methodContext2 
		initializeWith: 18 
		stackPtr: 1 
		method: FLPair >> #method2 
		receiver:  (FLPair new right: 5; left: 6) 
		sender: nil.

	methodContext1 privSender:  methodContext2.
		
	materializedMethodContext1 := self resultOfSerializeAndMaterialize: methodContext1.
	methodContext1 assertWellMaterializedInto: materializedMethodContext1 in: self.! !

!FLMethodContextSerializationTest methodsFor: 'tests' stamp: 'MartinDias 12/3/2011 22:22'!
testMethodContextWithTemp
	"This test should be improved"
	| methodContext1  materializedMethodContext1  |
	methodContext1 := (MethodContext newForMethod: FLPair >> #methodWithTemp).
	methodContext1 
		initializeWith: nil 
		stackPtr: 1 
		method: FLPair >> #methodWithTemp 
		receiver:  (FLPair new right: 4; left: 2) 
		sender: nil.
	methodContext1 tempNamed: 'string' put: 'capo'.
	
	materializedMethodContext1 := self resultOfSerializeAndMaterialize: methodContext1.
	self assert: (materializedMethodContext1 tempNamed: 'string') = 'capo'.
	methodContext1 assertWellMaterializedInto: materializedMethodContext1 in: self.! !

FLSerializationTest subclass: #FLPluggableSubstitutionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLPluggableSubstitutionTest methodsFor: 'tests' stamp: 'MartinDias 5/9/2012 00:17'!
testAvoidRecursion

	| result |
	self analyzer when: [:x | x < 10 ] substituteBy: [:x | x + 1 ].

	result := self resultOfSerializeAndMaterialize: 1.
	
	self assert: result = 2.! !

!FLPluggableSubstitutionTest methodsFor: 'tests' stamp: 'MartinDias 10/7/2011 16:33'!
testLowercaseSomeSymbols

	| result |
	self analyzer when: [:x | #A = x] substituteBy: [:x | #a].
	self analyzer when: [:x | #B = x] substituteBy: [:x | #b].

	result := self resultOfSerializeAndMaterialize: #(A B C).
	
	self assert: #(a b C) = result.! !

!FLPluggableSubstitutionTest methodsFor: 'tests' stamp: 'MartinDias 5/8/2012 20:03'!
testSimple

	| result |
	self analyzer when: [:x | x < 0 ] substituteBy: [:x | 0 ].

	result := self resultOfSerializeAndMaterialize: -1.
	
	self assert: result = 0.! !

!FLPluggableSubstitutionTest methodsFor: 'tests' stamp: 'MartinDias 5/8/2012 19:57'!
testSubstituteByItself

	| result |
	self analyzer when: [:x | true ] substituteBy: [:x | x ].

	result := self resultOfSerializeAndMaterialize: 1.
	
	self assert: result = 1.! !

!FLPluggableSubstitutionTest methodsFor: 'tests' stamp: 'MartinDias 10/7/2011 16:33'!
testTransientPair

	| result |
	self analyzer when: [:x | FLPair = x class] substituteBy: [:x | nil].

	result := self resultOfSerializeAndMaterialize: FLPair new.
	
	self assert: result isNil.! !

!FLPluggableSubstitutionTest methodsFor: 'tests' stamp: 'MartinDias 10/7/2011 16:33'!
testTransientPairLeft

	| result |
	self analyzer when: [:x | FLPair = x class and: [x left isNil not]] substituteBy: [:x | x copy left: nil].

	result := self resultOfSerializeAndMaterialize: (FLPair new left: $A; yourself).
	
	self assert: result left isNil.! !

!FLPluggableSubstitutionTest methodsFor: 'tests' stamp: 'MartinDias 5/8/2012 19:26'!
testUniqueSubstitution
	"Tests that an object with two occurrences in the graph is substituted to the *same* object."

	| result |
	self analyzer when: [ :x | x isNumber ] substituteBy: [ :x | x printString ].

	"This line demonstrates that behavior being tested is not trivial"
	self deny: 1 printString == 1 printString.

	result := self resultOfSerializeAndMaterialize: (Array with: 1 with: 1).
	
	self assert: result first = '1'.	
	self assert: result first == result second.! !

!FLSerializationTest class methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 10/31/2011 11:58'!
packageNamesUnderTest
	^ #('Fuel')! !

!FLSerializationTest methodsFor: 'accessing' stamp: 'MartinDias 10/7/2011 16:33'!
analyzer
	^serializer analyzer! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/22/2012 10:39'!
assertSerializationEqualityOf: anObject
	"Asserts that the original object and the materialized one are equal (but not the same)"

	| materialized |
	materialized := self resultOfSerializeAndMaterialize: anObject.
	self assert: anObject ~~ materialized description: 'The materialized object should not be the same as the serialized one'.
	self assert: anObject = materialized description: 'The materialized object should be equal to serialized one'.! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 11/29/2011 20:44'!
assertSerializationIdentityOf: anObject
	"Asserts that the original object and the materialized one are the same"

	| result |
	result := self resultOfSerializeAndMaterialize: anObject.
	self 
		assert: anObject == result 
		description: 'The materialized object is not the same as the serialized one'.! !

!FLSerializationTest methodsFor: 'accessing' stamp: 'MartinDias 10/7/2011 16:31'!
classFactory
	^classFactory! !

!FLSerializationTest methodsFor: 'traits-factory' stamp: 'MarianoMartinezPeck 4/19/2012 19:40'!
cleanUpTraits

	Smalltalk globals allTraits 
		select: [ :aTrait | aTrait name beginsWith: self traitNamePrefix ]
		thenDo: [ :aTrait | self removeFromSystem: aTrait ]! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 1/6/2012 11:07'!
materialization
	
	self streamFactory readStreamDo: [:aStream |
		^ self materializer materializeFrom: aStream ]! !

!FLSerializationTest methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 7/28/2012 15:03'!
materializationHeader
	
	self streamFactory readStreamDo: [:aStream |
		^ self materializer materializeHeaderFrom: aStream ]! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 1/6/2012 11:07'!
materialized
	
	^ self materialization root! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 1/6/2012 11:07'!
materializedObjects
	
	^ self materialization objects! !

!FLSerializationTest methodsFor: 'accessing' stamp: 'MartinDias 10/6/2011 23:44'!
materializer
	^materializer! !

!FLSerializationTest methodsFor: 'class-factory' stamp: 'MarianoMartinezPeck 4/19/2012 19:14'!
newClass
	^ self newSubclassOf: Object 
		instanceVariableNames: '' 
		classVariableNames: ''
! !

!FLSerializationTest methodsFor: 'class-factory' stamp: 'MarianoMartinezPeck 4/19/2012 19:28'!
newClassInCategory: aCategory
	^ self classFactory silentlyNewClassInCategory: aCategory! !

!FLSerializationTest methodsFor: 'class-factory' stamp: 'MarianoMartinezPeck 4/19/2012 19:19'!
newSubclassOf: aClass  instanceVariableNames: ivNamesString classVariableNames: classVarsString
	^ self classFactory silentlyNewSubclassOf: aClass instanceVariableNames: ivNamesString classVariableNames:  classVarsString! !

!FLSerializationTest methodsFor: 'traits-factory' stamp: 'MarianoMartinezPeck 4/19/2012 19:13'!
newTraitSuffixed: suffix

	^ Trait 
		named: (self traitNamePrefix, suffix) asSymbol
		uses: Array new 
		category: (self classFactory packageName, '-', self classFactory defaultCategoryPostfix) asSymbol! !

!FLSerializationTest methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 4/19/2012 19:39'!
removeFromSystem: aClassOrTrait
	aClassOrTrait removeFromSystem: false! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/12/2011 10:56'!
resultOfSerializeAndMaterialize: anObject

	self serialize: anObject.
	^ self materialized
! !

!FLSerializationTest methodsFor: '*FuelTests' stamp: 'MarianoMartinezPeck 4/20/2012 21:38'!
resultOfSerializeAndMaterializeCompiledMethod: aCompiledMethod
	| materialized |
	materialized := self resultOfSerializeAndMaterialize: aCompiledMethod.
	self assert: (materialized isEqualRegardlessTrailerTo: aCompiledMethod)! !

!FLSerializationTest methodsFor: 'running' stamp: 'MarianoMartinezPeck 5/11/2012 10:46'!
resultOfSerializeAndMaterializeMethod: aCompiledMethod
	self analyzer considerInternalMethod: aCompiledMethod. 
	self serialize: aCompiledMethod.
	^ self materialized
! !

!FLSerializationTest methodsFor: '*FuelTests' stamp: 'MarianoMartinezPeck 4/20/2012 21:35'!
resultOfSerializeAndMaterializeMethodDictionary: aMethodDictionary
	| materialized |
	materialized := self resultOfSerializeAndMaterialize: aMethodDictionary.
	self assert: (materialized isEqualRegardlessMethodsTrailerTo: aMethodDictionary)! !

!FLSerializationTest methodsFor: 'running' stamp: 'MarianoMartinezPeck 12/9/2011 20:28'!
serializationOf: anObject 
	self streamFactory writeStreamDo: [:aStream |
	^ serializer serialize: anObject on: aStream. ].
! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 1/6/2012 11:13'!
serializationOf: anObject includes: anotherObject 
	| serialization |
	self streamFactory writeStreamDo: [:aStream |
	serialization := serializer serialize: anObject on: aStream.
	^ serialization objects includes: anotherObject ].! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/12/2011 10:58'!
serialize: anObject

	self streamFactory writeStreamDo: [:aStream |
		self serializer serialize: anObject on: aStream ]! !

!FLSerializationTest methodsFor: 'accessing' stamp: 'MartinDias 10/6/2011 23:44'!
serializer
	^serializer! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 4/12/2012 17:21'!
setUp
	super setUp.
	self setUpClassFactory.
	self setUpSerializer.
	self setUpMaterializer.
	"You have several stream strategies available on 'stream-strategies' category."
	self useMemoryStream.
! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/7/2011 16:29'!
setUpClassFactory
	classFactory := ClassFactoryForTestCase new
! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/6/2011 23:52'!
setUpMaterializer
	materializer := FLMaterializer newDefault! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/6/2011 23:52'!
setUpSerializer
	serializer := FLSerializer newDefault! !

!FLSerializationTest methodsFor: 'accessing' stamp: 'MartinDias 10/12/2011 10:57'!
streamFactory
	^streamFactory! !

!FLSerializationTest methodsFor: 'running' stamp: 'MarianoMartinezPeck 4/16/2012 20:31'!
tearDown
	super tearDown.
	"Traits should be cleaned before classFactory because it seems class factory only knwos how to clean classes, not traits."
	self tearDownTraits.
	self tearDownClassFactory.
	self tearDownGlobalVariables.! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/8/2011 01:26'!
tearDownClassFactory
	self classFactory cleanUp! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 12/30/2011 19:03'!
tearDownGlobalVariables

	Smalltalk globals removeKey: #FLGlobalVariableForTesting ifAbsent: []! !

!FLSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/8/2011 04:05'!
tearDownTraits
	self cleanUpTraits! !

!FLSerializationTest methodsFor: 'traits-factory' stamp: 'MartinDias 10/8/2011 04:03'!
traitNamePrefix
	^'FLTraitForTesting'! !

!FLSerializationTest methodsFor: 'stream-strategies' stamp: 'MartinDias 10/12/2011 11:21'!
useInMemorySerializationStream 
	"This is a special factory to test what we offer with FLSerializer class >> #serializeInMemory:  and  FLMaterializer class >> #materializeFromByteArray: "
	streamFactory := FLByteArrayStreamStrategy new! !

!FLSerializationTest methodsFor: 'stream-strategies' stamp: 'MartinDias 10/12/2011 11:21'!
useMemoryStream

	streamFactory := FLMultiByteStreamStrategy new! !

!FLSerializationTest methodsFor: 'stream-strategies' stamp: 'MartinDias 12/7/2011 02:11'!
useMultiByteFileStream

	streamFactory := FLFileStreamStrategy newWithMultiByteFileStream! !

!FLSerializationTest methodsFor: 'stream-strategies' stamp: 'MartinDias 12/7/2011 02:11'!
useStandardFileStream

	streamFactory := FLFileStreamStrategy newWithStandardFileStream! !

!FLSerializationTest methodsFor: 'class-factory' stamp: 'MarianoMartinezPeck 4/19/2012 19:18'!
withNotificationsNewClass
	^ self classFactory withNotificationsNewClass! !

!FLSerializationTest methodsFor: 'class-factory' stamp: 'MarianoMartinezPeck 4/19/2012 19:30'!
withNotificationsNewClassWithInstanceVariableNames: instanceVariableNames
	"Returns a class for testing, with the specified instance variables."

	^ classFactory withNotificationsNewClassWithInstanceVariableNames: instanceVariableNames! !

FLSerializationTest subclass: #FLSignatureTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLSignatureTest methodsFor: 'tests' stamp: 'MartinDias 3/20/2012 12:59'!
testBadSignature

	serializer signature: 'FUELx'.
	materializer signature: 'FUELy'.
	self
		should: [ self resultOfSerializeAndMaterialize: 'content' ]
		raise: FLBadSignature
		whoseDescriptionIncludes: 'FUELy'
		description: 'The materializer should raise an error when signature differs of the expected one.'! !

!FLSignatureTest methodsFor: 'tests' stamp: 'MartinDias 10/7/2011 12:22'!
testSameSignature

	serializer signature: 'FUELx'.
	materializer signature: 'FUELx'.
	self assertSerializationEqualityOf: 'content'! !

FLSerializationTest subclass: #FLSortedCollectionSerializationTest
	instanceVariableNames: 'instanceVariableForTesting'
	classVariableNames: 'ClassVariableForTesting'
	poolDictionaries: ''
	category: 'FuelTests'!

!FLSortedCollectionSerializationTest class methodsFor: 'sorted collections for testing' stamp: 'MarianoMartinezPeck 6/8/2011 18:05'!
sortedCollectionForTestingWithClassVariable.
	^  SortedCollection sortBlock: [:a :b | 
		ClassVariableForTesting 
			ifTrue: [ a <= b ] 
			ifFalse: [ a >= b ] ].
! !

!FLSortedCollectionSerializationTest methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 6/8/2011 17:50'!
classVariableForTesting
	
	^ ClassVariableForTesting! !

!FLSortedCollectionSerializationTest methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 6/8/2011 17:50'!
classVariableForTesting: anObject
	
	ClassVariableForTesting := anObject ! !

!FLSortedCollectionSerializationTest methodsFor: 'running' stamp: 'MartinDias 3/26/2012 21:40'!
sortedBlockClosureWithInstanceVariable
	^ SortedCollection
		sortBlock: [ :a :b | 
			instanceVariableForTesting
				ifTrue: [ a <= b ]
				ifFalse: [ a >= b ] ]! !

!FLSortedCollectionSerializationTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 19:24'!
testSortedCollection
	"Tests that a SortedCollection works fine after materialization. 
	Note: Equality can not be used to assert, since its behavior is this:
	| x y |
	x := SortedCollection sortBlock: [:a :b | a >= b ].
	y := SortedCollection sortBlock: [:a :b | a >= b ].
	self assert: x ~= y"

	| aSortBlock aSortedCollection materialized |
	aSortBlock := [:a :b | a >= b ].
	aSortedCollection := SortedCollection sortBlock: aSortBlock.

	materialized := self resultOfSerializeAndMaterialize: aSortedCollection.
	self assert: aSortedCollection sortBlock ~~ materialized sortBlock.

	materialized addAll: #(2 1 3).
	aSortedCollection addAll: #(2 1 3).	
	self assert: aSortedCollection asArray = materialized asArray.! !

!FLSortedCollectionSerializationTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 6/8/2011 23:21'!
testSortedCollectionWithClassVariableChanges
	"Tests serialization of a SortedCollection whose sortBlock has a reference to a class variable and its value is changed. See also FLBlockClosureSerializationTest >> testBlockClosureWithClassVariableChanges  and FLCompiledMethodSerializationTest >> testMethodChangingClassVariable"

	| aSortedCollection materialized |
	ClassVariableForTesting := false.
	
	aSortedCollection := self class sortedCollectionForTestingWithClassVariable.
	materialized := self resultOfSerializeAndMaterialize: aSortedCollection.
	"the class variable ClassVariableForTesting should be false"
	self deny: (materialized sortBlock outerContext method literalAt: 3) value.
	ClassVariableForTesting := true.
	"the class variable ClassVariableForTesting should be true"
	self assert: (materialized sortBlock outerContext method literalAt: 3) value.
	
! !

!FLSortedCollectionSerializationTest methodsFor: 'tests' stamp: 'mariano 12/4/2011 18:57'!
testSortedCollectionWithClassVariableReference
	"Tests serialization of a SortedCollection whose sortBlock has a reference to the outer context. 
	Note: Equality can not be used to assert, since its behavior is this:
	| x y |
	x := SortedCollection sortBlock: [:a :b | a >= b ].
	y := SortedCollection sortBlock: [:a :b | a >= b ].
	self assert: x ~= y"

	| aSortedCollection materialized |
	ClassVariableForTesting := false.
	
	aSortedCollection := self class sortedCollectionForTestingWithClassVariable.
	materialized := self resultOfSerializeAndMaterialize: aSortedCollection.
	ClassVariableForTesting := true.
	materialized addAll: #(2 3 1).
	aSortedCollection addAll: #(2 3 1).
	
	aSortedCollection sortBlock assertWellMaterializedInto:  materialized sortBlock in: self.
	self assert: aSortedCollection asArray = materialized asArray.! !

!FLSortedCollectionSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/26/2012 22:29'!
testSortedCollectionWithInstanceVariableReference
	"Tests serialization of a SortedCollection whose sortBlock has a reference to the outer context. 
	Note: Equality can not be used to assert, since its behavior is this:
	| x y |
	x := SortedCollection sortBlock: [:a :b | a >= b ].
	y := SortedCollection sortBlock: [:a :b | a >= b ].
	self assert: x ~= y"

	| aSortedCollection materialized |
	instanceVariableForTesting := false.
	aSortedCollection := self sortedBlockClosureWithInstanceVariable.
	
	materialized := self resultOfSerializeAndMaterialize: aSortedCollection.
	materialized addAll: #(2 3 1).
	aSortedCollection addAll: #(2 3 1).
	
	self assert: aSortedCollection asArray = materialized asArray.
	self assert: aSortedCollection asArray = #(3 2 1)! !

!FLSortedCollectionSerializationTest methodsFor: 'tests' stamp: 'MartinDias 3/26/2012 22:28'!
testSortedCollectionWithInstanceVariableReferenceChange
	"IMPORTANT: this test is to demonstrate a limitation.  Unfortunatly there is nothing we can do to solve this problem. The sortBlock was serialized with a method context with different receiver, different temps, etc.
	Tests serialization of a SortedCollection whose sortBlock has a reference to the outer context. 
	Note: Equality can not be used to assert, since its behavior is this:
	| x y |
	x := SortedCollection sortBlock: [:a :b | a >= b ].
	y := SortedCollection sortBlock: [:a :b | a >= b ].
	self assert: x ~= y"

	| aSortedCollection materialized |
	instanceVariableForTesting := false.
	aSortedCollection := self sortedBlockClosureWithInstanceVariable.

	materialized := self resultOfSerializeAndMaterialize: aSortedCollection.

	instanceVariableForTesting := true.
	materialized addAll: #(2 3 1).
	aSortedCollection addAll: #(2 3 1).

	"IMPORTANT: Unfortunatly there is nothing we can do to solve this problem. The sortBlock was serialized with a method context with different values."
	self deny: aSortedCollection asArray = materialized asArray.! !

FLSerializationTest subclass: #FLTraitSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLTraitSerializationTest commentStamp: '<historical>' prior: 0!
I have the common behavior for testing trait serialization.!

FLTraitSerializationTest subclass: #FLGlobalTraitSerializationTest
	uses: FLTGlobalClassOrTraitSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLGlobalTraitSerializationTest commentStamp: '<historical>' prior: 0!
I test the serialization of traits as *external* objects, i.e. the traits has to be present in the image at materialization time.!

FLGlobalTraitSerializationTest class
	uses: FLTGlobalClassOrTraitSerializationTest classTrait
	instanceVariableNames: ''!

!FLTraitSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/8/2011 04:14'!
newClassOrTrait
	"Returns a trait for testing"

	^ self newTraitSuffixed: 'Main'! !

!FLTraitSerializationTest methodsFor: 'running' stamp: 'MarianoMartinezPeck 4/19/2012 19:13'!
newInstanceFrom: aTrait

	| aClass |
	aClass := self newClass.
	"If one wants to use a materialized trait in a class, it has to be added to Smalltalk globals. Fuel does not do that. Fuel just materializes the trait. It is up to the user to decide what to do with it.  "
	Smalltalk globals at: aTrait name put: aTrait.
	aClass addToComposition: aTrait.
	^ aClass new! !

!FLTraitSerializationTest methodsFor: 'running' stamp: 'MartinDias 10/8/2011 04:14'!
newSecondaryTrait
	"Returns a trait for testing"

	^ self newTraitSuffixed: 'Secondary'! !

FLSerializationTest subclass: #FLVersionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLVersionTest methodsFor: 'tests' stamp: 'MartinDias 3/20/2012 12:59'!
testBadVersion

	serializer version: 2.
	materializer version: 1.
	self
		should: [ self resultOfSerializeAndMaterialize: 'content' ]
		raise: FLBadVersion
		whoseDescriptionIncludes: '2'
		description: 'The materializer should raise an error when version differs of the expected one.'! !

!FLVersionTest methodsFor: 'tests' stamp: 'MartinDias 10/7/2011 12:24'!
testSameVersion

	serializer version: 1.
	materializer version: 1.
	self assertSerializationEqualityOf: 'content'! !

FLSerializationTest subclass: #FLWeakObjectsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLWeakObjectsTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 12/6/2011 19:16'!
testAssociationWithWeak
	"This tests an association with weak values. InstVar1 is a fixed instance variable, hence they are strong and hence should not be replaed by nil. Instead, 'weak' is stored in the variable part of FLWeakClassMock, so it is weak and so it should have been replaced by nil."
	| weak association materializedAssociation |
	
	weak := FLWeakClassMock new: 1.
	weak instVar1: 'nonWeak'.
	weak basicAt: 1 put: 'weak'. 
	association := Association key: #foo value: weak.
	
	materializedAssociation := self resultOfSerializeAndMaterialize: association.
	self assert: materializedAssociation value instVar1 isNil not.
	self assert: (materializedAssociation value basicAt: 1) isNil.
	self assert: materializedAssociation key = #foo.
	! !

!FLWeakObjectsTest methodsFor: 'tests' stamp: 'MartinDias 3/22/2012 22:54'!
testAssociationWithWeakFinalizationList
	"This tests an association with WeakFInalizationList as its value."
	| weak association materializedAssociation object list |
	
	object := Object new.
	list := WeakFinalizationList new.
	weak := WeakFinalizerItem new list: list object: object executor: nil.
	association := Association key: #foo value: weak.
	
	materializedAssociation := self resultOfSerializeAndMaterialize: association.
	"Both, 'executor' and 'list' are fixed instance variables, hence they are strong and hence should not be replaced by nil. Instead, 'object' is stored in the variable part of WeakFinalizationItem, so it is weak and so it should have been replaced by nil."
	self assert: materializedAssociation value executor isNil.
	self assert: materializedAssociation value list isNil not.
	self assert: materializedAssociation value object isNil.
	self assert: materializedAssociation key = #foo.
	! !

!FLWeakObjectsTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 12/6/2011 12:11'!
testWeakAndNoStrongReferenceFromGraph
	"This tests when there are weak objects inside the graph to serialize but there are no strong references from the graph."
	| weak graph obj1 obj2 materializedGraph |
	obj1 := 11.
	obj2 := 'something'.
	weak := WeakArray with: obj1 with: obj2. 
	graph := OrderedCollection with: 40 with: 'aaaa' with: weak.
	
	materializedGraph := self resultOfSerializeAndMaterialize: graph.
	self assert: (materializedGraph at: 3) first isNil.
	self assert: (materializedGraph at: 3) second isNil.
	
	self deny: (self serializationOf: graph includes: 11).
	self deny: (self serializationOf: graph includes: 'something').
	
	 
		
! !

!FLWeakObjectsTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 12/6/2011 12:11'!
testWeakWithStrongReferenceFromGraph
	"This tests when there are weak objects inside the graph to serialize and there are also strong references from the graph."
	| weak graph obj1 obj2 materializedGraph |
	obj1 := 11.
	obj2 := 'something'.
	weak := WeakArray with: obj1 with: obj2. 
	graph := OrderedCollection with: 40 with: 'aaaa' with: weak with: obj2.
	
	materializedGraph := self resultOfSerializeAndMaterialize: graph.
	self assert: (materializedGraph at: 3) first isNil.
	self assert: (materializedGraph at: 3) second = obj2.
	
	self deny: (self serializationOf: graph includes: 11).
	self assert: (self serializationOf: graph includes: obj2).
	
	 
		

! !

!FLWeakObjectsTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 12/6/2011 13:59'!
testWeakWithStrongReferenceFromGraph2
	"This tests when there are weak objects inside the graph to serialize and there are also strong references from the graph."
	| weak graph obj1 obj2 materializedGraph |
	obj1 := 11.
	obj2 := 'something'.
	weak := WeakArray with: obj1 with: obj2. 
	graph := OrderedCollection with: 40 with: 'aaaa' with: weak with: obj2 with: (Point x: weak y: weak).
	 
	materializedGraph := self resultOfSerializeAndMaterialize: graph.
	self assert: (materializedGraph at: 3) first isNil.
	self assert: (materializedGraph at: 3) second = obj2.
	
	self deny: (self serializationOf: graph includes: 11).
	self assert: (self serializationOf: graph includes: obj2).
	
	 
		

! !

TestCase subclass: #FLSimpleStackTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests-Collections'!

!FLSimpleStackTest methodsFor: 'testing' stamp: 'MartinDias 10/5/2011 19:23'!
testCapacity

	| stack |
	stack := FLSimpleStack new: 3.
	self assert: 3 = stack capacity.
	
	stack := FLSimpleStack new: 5.
	self assert: 5 = stack capacity.! !

!FLSimpleStackTest methodsFor: 'tests-adding' stamp: 'MartinDias 5/15/2011 23:13'!
testGrow

	| stack |
	stack := FLSimpleStack new: 1.
	self assert: 1 = stack capacity.
	stack push: #a.
	self assert: 1 < stack capacity.! !

!FLSimpleStackTest methodsFor: 'testing' stamp: 'MartinDias 5/15/2011 22:50'!
testIsEmpty

	| stack |
	stack := FLSimpleStack new.
	self assert: stack isEmpty.! !

!FLSimpleStackTest methodsFor: 'testing' stamp: 'MartinDias 5/15/2011 22:50'!
testPop

	| stack |
	stack := FLSimpleStack new.
	stack push: 1.
	stack pop.
	self assert: stack isEmpty.! !

!FLSimpleStackTest methodsFor: 'testing' stamp: 'MartinDias 5/15/2011 22:50'!
testPush

	| stack |
	stack := FLSimpleStack new.
	stack push: 1.
	self deny: stack isEmpty.! !

TestCase subclass: #FLUserGuidesTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests'!

!FLUserGuidesTest methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 6/12/2012 22:40'!
tearDown
	super tearDown.
	#('demo.fuel' 'numbers.fuel' 'number.fuel.zip' 'example.FL' ) 
		do: [:each | FileDirectory default deleteFileNamed: each].
		! !

!FLUserGuidesTest methodsFor: 'tests-Managing Globals' stamp: 'MartinDias 5/19/2012 01:10'!
testConsiderGlobal
	
	| aSerializer anArray materializedArray |
	"Prepare an array whose two elements are system globals."
	anArray := Array with: Set new with: Set new.
	Smalltalk at: #GlobalSet1 put: anArray first.
	Smalltalk at: #GlobalSet2 put: anArray second.

	"Serialize considering *only first* as a global object."
	FileStream forceNewFileNamed: 'demo.fuel' do: [:aStream |
		aSerializer := FLSerializer newDefault.
		aSerializer analyzer considerGlobal: #GlobalSet1.
		aSerializer serialize: anArray on: aStream binary].

	"Materialize"
	FileStream oldFileNamed: 'demo.fuel' do: [:aStream |
		materializedArray := (FLMaterializer newDefault 
			materializeFrom: aStream binary) root].
		
	"Check that second element is a new Set."
	[ (Smalltalk at: #GlobalSet1) == materializedArray first ] assert.
	[ (Smalltalk at: #GlobalSet2) ~~ materializedArray second ] assert.
! !

!FLUserGuidesTest methodsFor: 'tests-Getting Started' stamp: 'MartinDias 5/18/2012 19:29'!
testDemo

| sourceArray loadedArray |
sourceArray := 
	Array 
		with: 'a string' 
		with: Transcript
		with: [ Transcript show: 'a string'; flush ].
"Store to the file"
FLSerializer serialize: sourceArray toFileNamed: 'example.FL'. 
"Load from the file"
loadedArray := FLMaterializer materializeFromFileNamed: 'example.FL'. 
"The arrays are not the same"
[ sourceArray ~~ loadedArray ] assert.
"The strings are not the same"
[ sourceArray first ~~ loadedArray first ] assert.
[ sourceArray first = loadedArray first ] assert.
"The global instance Transcript is the same"
[ sourceArray second == loadedArray second ] assert.
"Look at Transcript how the loaded block prints a string"
loadedArray third value.! !

!FLUserGuidesTest methodsFor: 'tests-Getting Started' stamp: 'MartinDias 5/18/2012 14:33'!
testFileShortcuts

| materializedString |
FLSerializer serialize: 'stringToSerialize' toFileNamed: 'demo.fuel'. 
materializedString := FLMaterializer materializeFromFileNamed: 'demo.fuel'.
! !

!FLUserGuidesTest methodsFor: 'tests-Getting Started' stamp: 'MartinDias 5/19/2012 00:48'!
testFileStream

	| materializedString |
	FileStream forceNewFileNamed: 'demo.fuel' do: [:aStream |
		FLSerializer newDefault 
			serialize: 'stringToSerialize' 
	 		on: aStream binary].

	FileStream oldFileNamed: 'demo.fuel' do: [:aStream |
		materializedString := (FLMaterializer newDefault 
			materializeFrom: aStream binary) root].! !

!FLUserGuidesTest methodsFor: 'tests-Getting Started' stamp: 'MartinDias 5/18/2012 15:39'!
testGZip

	| materialization |
	FileStream forceNewFileNamed: 'number.fuel.zip' do: [:aFileStream | 
		|gzip|
		aFileStream binary.
		gzip := GZipWriteStream on: aFileStream.
		FLSerializer newDefault serialize: 123 on: gzip.
		gzip close. ].
	
	FileStream oldFileNamed: 'number.fuel.zip' do: [:aFileStream | 
		|gzip|
		aFileStream binary.
		gzip := GZipReadStream on: aFileStream.		
		materialization := FLMaterializer newDefault 
			materializeFrom: gzip.
		gzip close. ].! !

!FLUserGuidesTest methodsFor: 'tests-Getting Started' stamp: 'MartinDias 5/18/2012 14:32'!
testMemoryStream

| anArray materializedString |
anArray := FLSerializer serializeToByteArray: 'stringToSerialize'. 
materializedString := FLMaterializer materializeFromByteArray: anArray.

! !

!FLUserGuidesTest methodsFor: 'tests-Customizing Graph' stamp: 'MartinDias 5/24/2012 11:06'!
testPluggableSubstitution

	| materialized objectToSerialize aSerializer |
	
	objectToSerialize := Array with: 'hello' with: '' writeStream.
	
	FileStream forceNewFileNamed: 'demo.fuel' do: [ :aStream |
		aSerializer := FLSerializer newDefault.
		aSerializer analyzer when: [ :o | o isStream ] substituteBy: [ :o | nil ].
		aSerializer		 
			serialize: objectToSerialize
	 		on: aStream binary ].

	FileStream oldFileNamed: 'demo.fuel' do: [ :aStream |
		(FLMaterializer newDefault 
			materializeFrom: aStream binary) 
			root ].! !

!FLUserGuidesTest methodsFor: 'tests-Getting Started' stamp: 'MartinDias 5/18/2012 15:34'!
testShowingProgressBar
	| materializedString |
	(Smalltalk globals includesKey: #FLProgressSerialization) ifFalse: [
		" Needs:
		(ConfigurationOfFuel project latestVersion) load: 'FuelProgressUpdate'.
		"
		^self ].
	
	FileStream forceNewFileNamed: 'numbers.fuel' do: [:aStream |
		FLSerializer newDefault 
			showProgress;
			serialize: (1 to: 200000) asArray 
			on: aStream binary ].
	
	FileStream oldFileNamed: 'numbers.fuel' do: [:aStream | 
		FLMaterializer newDefault 
			showProgress;
			materializeFrom: aStream binary ].! !

!Class methodsFor: '*FuelTests' stamp: 'EstebanLorenzano 7/27/2012 16:30'!
renameSilently: aName
	SystemAnnouncer uniqueInstance doSilently: [
		self rename: aName.
	]! !

Object subclass: #FLClassWithRecursiveSubstitution
	instanceVariableNames: 'index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests-Mocks'!

!FLClassWithRecursiveSubstitution methodsFor: 'serialization' stamp: 'MartinDias 5/9/2012 00:57'!
fuelAccept: aGeneralMapper

	^ index < 10
		ifTrue: [
			aGeneralMapper 
				visitSubstitution: self 
				by: (self copy index: self index + 1)
				onRecursionDo: [ super fuelAccept: aGeneralMapper ] ]
		ifFalse: [
			super fuelAccept: aGeneralMapper ]! !

!FLClassWithRecursiveSubstitution methodsFor: 'accessing' stamp: 'MartinDias 5/9/2012 00:48'!
index
	^ index! !

!FLClassWithRecursiveSubstitution methodsFor: 'accessing' stamp: 'MartinDias 5/9/2012 00:47'!
index: anObject
	index := anObject ! !

Object subclass: #FLGlobalSendMock
	instanceVariableNames: 'contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests-Mocks'!

!FLGlobalSendMock class methodsFor: 'instance creation' stamp: 'MartinDias 9/15/2011 02:57'!
newInstanceToMaterialize
	
	^self newWith: #materializing! !

!FLGlobalSendMock class methodsFor: 'instance creation' stamp: 'MartinDias 9/15/2011 02:55'!
newInstanceToSerialize
	
	^self newWith: #serializing! !

!FLGlobalSendMock class methodsFor: 'instance creation' stamp: 'MartinDias 9/15/2011 02:56'!
newWith: aByteSymbol 
	
	^self basicNew
		initializeWith: aByteSymbol;
		yourself! !

!FLGlobalSendMock methodsFor: 'comparing' stamp: 'MartinDias 9/15/2011 02:58'!
= anObject
	"Answer whether the receiver and anObject represent the same object."

	self == anObject
		ifTrue: [ ^ true ].
	self class = anObject class
		ifFalse: [ ^ false ].
	^ contents = anObject contents! !

!FLGlobalSendMock methodsFor: 'accessing' stamp: 'MartinDias 9/15/2011 02:58'!
contents
	^ contents! !

!FLGlobalSendMock methodsFor: 'accessing' stamp: 'MartinDias 9/15/2011 02:58'!
contents: anObject
	contents := anObject! !

!FLGlobalSendMock methodsFor: 'serialization' stamp: 'MartinDias 3/29/2012 03:05'!
fuelAccept: aGeneralMapper

	^aGeneralMapper visitGlobalSend: self name: #FLGlobalSendMock selector: #newInstanceToMaterialize! !

!FLGlobalSendMock methodsFor: 'comparing' stamp: 'MartinDias 9/15/2011 02:58'!
hash
	"Answer an integer value that is related to the identity of the receiver."

	^ contents hash! !

!FLGlobalSendMock methodsFor: 'initialization' stamp: 'MartinDias 9/15/2011 02:56'!
initializeWith: aByteSymbol
	
	super initialize.
	contents := aByteSymbol ! !

!FLGlobalSendMock methodsFor: 'printing' stamp: 'MartinDias 9/15/2011 02:59'!
printOn: aStream
	"Append a sequence of characters to aStream that identify the receiver."

	super printOn: aStream.
	aStream
		nextPutAll: ' contents: ';
		print: contents! !

Object subclass: #FLGlobalSendNotPresentMock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests-Mocks'!

!FLGlobalSendNotPresentMock methodsFor: 'serialization' stamp: 'MartinDias 3/29/2012 03:05'!
fuelAccept: aGeneralMapper

	^aGeneralMapper visitGlobalSend: self name: #FLKeyNotPresentInSmalltalks selector: #someSelector! !

Object subclass: #FLNotSerializableMock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests-Mocks'!

!FLNotSerializableMock methodsFor: 'hooks' stamp: 'MartinDias 5/18/2012 17:37'!
fuelAccept: aGeneralMapper

	aGeneralMapper visitNotSerializable: self! !

Object subclass: #FLPair
	instanceVariableNames: 'left right'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests-Mocks'!

!FLPair methodsFor: 'comparing' stamp: 'MartinDias 10/15/2010 10:54'!
= anObject
	"Answer whether the receiver and anObject represent the same object."

	self == anObject
		ifTrue: [ ^ true ].
	self class = anObject class
		ifFalse: [ ^ false ].
	^ self right = anObject right and: [ self left = anObject left ]! !

!FLPair methodsFor: 'comparing' stamp: 'MartinDias 10/15/2010 10:54'!
hash
	"Answer an integer value that is related to the identity of the receiver."

	^ self right hash bitXor: self left hash! !

!FLPair methodsFor: 'accessing' stamp: 'MartinDias 9/16/2010 19:58'!
left
	^ left! !

!FLPair methodsFor: 'accessing' stamp: 'MartinDias 9/16/2010 19:58'!
left: anObject
	left := anObject! !

!FLPair methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/23/2011 00:14'!
method1
	self name.
	self printString.
	! !

!FLPair methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/23/2011 00:14'!
method2
	self name.
	! !

!FLPair methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 7/1/2011 11:17'!
methodWithTemp
	| string |
	string := 'test'.
	self name.
	self printString.
	! !

!FLPair methodsFor: 'printing' stamp: 'MartinDias 8/20/2011 22:57'!
printOn: aStream
	"Append a sequence of characters to aStream that identify the receiver."

	super printOn: aStream.
	aStream
		nextPutAll: '(';
		print: left;
		space;
		print: right;
		nextPutAll: ')'! !

!FLPair methodsFor: 'accessing' stamp: 'MartinDias 9/16/2010 19:58'!
right
	^ right! !

!FLPair methodsFor: 'accessing' stamp: 'MartinDias 9/16/2010 19:58'!
right: anObject
	right := anObject! !

FLPair weakSubclass: #FLWeakClassMock
	instanceVariableNames: 'instVar1'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests-Mocks'!

!FLWeakClassMock methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 12/6/2011 21:23'!
instVar1
	^ instVar1 ! !

!FLWeakClassMock methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 12/6/2011 21:22'!
instVar1: anObject
	instVar1 := anObject! !

Object subclass: #FLPerson
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests-Mocks'!

!FLPerson commentStamp: 'MarianoMartinezPeck 5/19/2011 23:51' prior: 0!
FLPerson is a class just to be able to change the hash of an object and try to test problems with hash.!

!FLPerson methodsFor: 'comparing' stamp: 'MarianoMartinezPeck 5/19/2011 22:29'!
= anObject
	"Answer whether the receiver and anObject represent the same object."

	self == anObject
		ifTrue: [ ^ true ].
	self class = anObject class
		ifFalse: [ ^ false ].
	^ id = anObject id! !

!FLPerson methodsFor: 'comparing' stamp: 'MarianoMartinezPeck 5/19/2011 22:29'!
hash
	"Answer an integer value that is related to the identity of the receiver."

	^ id! !

!FLPerson methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/19/2011 22:28'!
id
	^ id! !

!FLPerson methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/19/2011 22:28'!
id: anObject
	id := anObject! !

Object subclass: #FLProxyThatBecomesItsContent
	instanceVariableNames: 'contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests-Mocks'!

!FLProxyThatBecomesItsContent commentStamp: 'MartinDias 11/17/2011 03:18' prior: 0!
Used at #testObjectByProxyThatBecomesItsContent!

!FLProxyThatBecomesItsContent class methodsFor: 'instance creation' stamp: 'MartinDias 11/17/2011 02:59'!
newWith: aByteSymbol 
	
	^self basicNew
		initializeWith: aByteSymbol;
		yourself! !

!FLProxyThatBecomesItsContent methodsFor: 'serialization' stamp: 'MartinDias 11/17/2011 03:03'!
fuelAfterMaterialization 

	self become: contents! !

!FLProxyThatBecomesItsContent methodsFor: 'initialization' stamp: 'MartinDias 11/17/2011 03:04'!
initializeWith: anObject
	
	super initialize.
	contents := anObject! !

Object subclass: #FLStreamStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests-StreamStrategies'!

!FLStreamStrategy commentStamp: 'MartinDias 10/12/2011 11:36' prior: 0!
I am a strategy that help tests for writing (and then reading) on streams.!

FLStreamStrategy subclass: #FLByteArrayStreamStrategy
	instanceVariableNames: 'inMemoryStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests-StreamStrategies'!

!FLByteArrayStreamStrategy commentStamp: 'MartinDias 10/12/2011 11:37' prior: 0!
I am a strategy that emulate what we offer with FLSerializer class >> #serializeInMemory:  and  FLMaterializer class >> #materializeFromByteArray: !

!FLByteArrayStreamStrategy methodsFor: 'reading' stamp: 'MartinDias 10/12/2011 10:21'!
readStreamDo: aValuable
	"Evaluates the argument with a read stream. Answers the result."
	
	^aValuable value: inMemoryStream contents readStream! !

!FLByteArrayStreamStrategy methodsFor: 'writing' stamp: 'MartinDias 10/12/2011 10:20'!
writeStreamDo: aValuable
	"Evaluates the argument with a write stream. Answers the result."
	
	inMemoryStream := ByteArray new writeStream.
	^aValuable value: inMemoryStream binary
! !

FLStreamStrategy subclass: #FLFileStreamStrategy
	instanceVariableNames: 'fileStreamClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests-StreamStrategies'!

!FLFileStreamStrategy commentStamp: 'MartinDias 10/12/2011 11:37' prior: 0!
I am a strategy for traditional file streams.!

!FLFileStreamStrategy class methodsFor: 'instance creation' stamp: 'MartinDias 12/7/2011 02:05'!
new
	self error: 'Invalid creation method.'! !

!FLFileStreamStrategy class methodsFor: 'instance creation' stamp: 'MartinDias 12/7/2011 02:03'!
newWith: aFileStream

	^self basicNew
		initializeWith: aFileStream;
		yourself! !

!FLFileStreamStrategy class methodsFor: 'instance creation' stamp: 'MartinDias 12/7/2011 02:03'!
newWithMultiByteFileStream

	^self newWith: MultiByteFileStream! !

!FLFileStreamStrategy class methodsFor: 'instance creation' stamp: 'MartinDias 12/7/2011 02:02'!
newWithStandardFileStream

	^self newWith: StandardFileStream! !

!FLFileStreamStrategy methodsFor: 'writing' stamp: 'MarcusDenker 7/10/2012 10:35'!
fileName
	"Answer a filename to serialize and materialize using it."

	^ Smalltalk image shortImageName, '-tests.fuel'! !

!FLFileStreamStrategy methodsFor: 'writing' stamp: 'MartinDias 12/7/2011 02:01'!
fileStreamClass
	"Returns the FileStream specific class"
	
	^fileStreamClass! !

!FLFileStreamStrategy methodsFor: 'writing' stamp: 'MartinDias 12/7/2011 02:02'!
initializeWith: aFileStreamClass

	self initialize.	
	fileStreamClass := aFileStreamClass.! !

!FLFileStreamStrategy methodsFor: 'reading' stamp: 'MartinDias 12/7/2011 01:59'!
readStreamDo: aValuable
	"Evaluates the argument with a read stream. Answers the result."
	
	^self fileStreamClass
		oldFileNamed: self fileName
		do: [ :aStream | aValuable value: aStream binary ]! !

!FLFileStreamStrategy methodsFor: 'writing' stamp: 'MartinDias 12/7/2011 01:59'!
writeStreamDo: aValuable
	"Evaluates the argument with a write stream. Answers the result."

	^self fileStreamClass
		forceNewFileNamed: self fileName 
		do: [ :aStream | aValuable value: aStream binary ]! !

FLStreamStrategy subclass: #FLMultiByteStreamStrategy
	instanceVariableNames: 'inMemoryStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelTests-StreamStrategies'!

!FLMultiByteStreamStrategy commentStamp: 'MartinDias 10/12/2011 11:37' prior: 0!
I am a strategy for MultiByteBinaryOrTextStream..!

!FLMultiByteStreamStrategy methodsFor: 'reading' stamp: 'MartinDias 10/12/2011 10:25'!
readStreamDo: aValuable
	"Evaluates the argument with a read stream. Answers the result."
	
	^aValuable value: inMemoryStream reset! !

!FLMultiByteStreamStrategy methodsFor: 'writing' stamp: 'MartinDias 10/12/2011 10:25'!
writeStreamDo: aValuable
	"Evaluates the argument with a write stream. Answers the result."

	inMemoryStream := MultiByteBinaryOrTextStream on: ''.
	^aValuable value: inMemoryStream binary
! !

!FLStreamStrategy methodsFor: 'serializing' stamp: 'MartinDias 3/29/2012 03:05'!
fuelAccept: aGeneralMapper
	"Visit myself as a substitution. See #fuelSubstitution."

	^ aGeneralMapper visitSubstitution: self by: nil! !

!FLStreamStrategy methodsFor: 'reading' stamp: 'MartinDias 10/12/2011 10:18'!
readStreamDo: aValuable
	"Evaluates the argument with a read stream. Answer the result."
	
	self subclassResponsibility ! !

!FLStreamStrategy methodsFor: 'writing' stamp: 'MartinDias 10/12/2011 10:18'!
writeStreamDo: aValuable
	"Evaluates the argument with a write stream. Answer the result."
	
	self subclassResponsibility ! !
